<?php
/***********************************************************************
| Avactis (TM) Shopping Cart software developed by Pentasoft Corp.
| http://www.avactis.com
| -----------------------------------------------------------------------
| All source codes & content (c) Copyright 2004-2010, Pentasoft Corp.
| unless specifically noted otherwise.
| =============================================
| This source code is released under the Avactis License Agreement.
| The latest version of this license can be found here:
| http://www.avactis.com/license.php
|
| By using this software, you acknowledge having read this license agreement
| and agree to be bound thereby.
|
 ***********************************************************************/
?><?php

/**
 * @package CMS
 * @author Sergey Kulitsky
 *
 */

class CMSMenu
{
    function CMSMenu()
    {
        global $application;

        $this -> mTmplFiller = new TemplateFiller();

        $this -> _templates = array(
            'container' => 'CMSMenuContainer',
            'item'      => 'CMSMenuItem'
        );

        $this -> NoView = false;
        if ($application -> issetBlockTagFatalErrors('CMSMenu'))
            $this -> NoView = true;
    }

    function getTemplateFormat()
    {
        $format = array(
            'layout-file' => 'cms-menu-block.ini',
            'files'       => array(
                'CMSMenuContainer' => TEMPLATE_FILE_SIMPLE,
                'CMSMenuItem'      => TEMPLATE_FILE_SIMPLE
            ),
            'options'     => array(
            )
        );
        return $format;
    }

    /**
     * The output of the Viewer
     * Params: the following optional params can be accepted
     *      0: menu_id/menu_system_name:
     *      1: templates to be used for output
     */
    function output()
    {
        global $application;

        if ($this -> NoView)
            return '';

        $menu_id = 0;
        $template_name = '';

        // getting the menu_id from the params if any
        if (func_num_args() > 0)
            $menu_id = func_get_arg(0);

        // getting the templates if specified
        if (func_num_args() > 1)
            $template_name = func_get_arg(1);

        // getting the menu data
        $this -> _Menu_Data = modApiFunc('CMS', 'searchMenu',
                                         array('menu_id' => $menu_id,
                                               'menu_index' => $menu_id));

        // getting review data
        if (!empty($this -> _Menu_Data))
        {
            // the menu_id is specified and valid
            $this -> _Menu_Data = array_pop($this -> _Menu_Data);
            $menu_id = $this -> _Menu_Data['menu_id'];
            $this -> _Menu_Data['items'] = modApiFunc('CMS', 'getMenuItems',
                                                      $menu_id);
        }
        else
        {
            // invalid menu
            return '';
        }

        // setting the template engine
        $template_block = $application -> getBlockTemplate('CMSMenu');
        if ($template_name)
        {
            // assuming the template_name is the directory name
            $tmp = explode('/', $template_block['template']['directory']);
            array_pop($tmp);
            $tmp[] = $template_name;
            $template_block['template']['directory'] = implode('/', $tmp);
        }
        $this -> mTmplFiller -> setTemplate($template_block);

        // registering tags
        $_tags = array(
            'LocalMenuTitle' => $this -> _Menu_Data['menu_name'],
            'LocalMenuItems' => $this -> outputMenuItems()
        );
        $this -> _Template_Contents = $_tags;
        $application -> registerAttributes($this -> _Template_Contents);

        return $this -> mTmplFiller -> fill($this -> _templates['container']);
    }

    /**
     * Outputs menu items
     */
    function outputMenuItems()
    {
        global $application;

        $result = '';

        if (!is_array($this -> _Menu_Data['items'])
            || empty($this -> _Menu_Data['items']))
            return $result;

        foreach($this -> _Menu_Data['items'] as $k => $item)
        {
            $_tags = array(
                'LocalMenuItemLink'  => $this -> getMenuItemLink($item),
                'LocalMenuItemName'  => $item['item_name'],
                'LocalMenuItemType'  => $item['item_type'],
                'LocalMenuItemFirst' => ($k == 0),
                'LocalMenuItemLast'  => ($k == count($this -> _Menu_Data['items']) - 1)
            );
            $this -> _Template_Contents = $_tags;
            $application -> registerAttributes($this -> _Template_Contents);

            $result .= $this -> mTmplFiller -> fill($this -> _templates['item']);
        }

        return $result;
    }

    /**
     * Generates the link for menu item
     */
    function getMenuItemLink($item)
    {
        global $application;

        $result = '';

        switch($item['item_type'])
        {
            case CMS_MENU_ITEM_TYPE_TEXT:
                $result = '';
                break;

            case CMS_MENU_ITEM_TYPE_URL:
            case CMS_MENU_ITEM_TYPE_EXTERNAL_URL:
                $result = $item['item_link'];
                break;

            case CMS_MENU_ITEM_TYPE_STATIC_PAGE:
                loadClass('CCMSPageInfo');
                $CMSPage = new CCMSPageInfo($item['item_link']);
                $result = $CMSPage -> getCMSPageTagValue('link', array());
                break;

            case CMS_MENU_ITEM_TYPE_SYSTEM_PAGE:
                $r = new Request();
                $r -> setView($application -> getViewBySection($item['item_link']));
                if (_ml_strtolower($item['item_link']) == 'productlist')
                {
                    $r -> setCategoryID($item['param1']);
                    $r -> setAction('SetCurrCat');
                    $r -> setKey('category_id', $item['param1']);
                }
                if (_ml_strtolower($item['item_link']) == 'productinfo')
                {
                    loadClass('CProductInfo');
                    $pr = new CProductInfo($item['param2']);
                    $item['param1'] = $pr -> chooseCategoryID();
                    $r -> setCategoryID($item['param1']);
                    $r -> setProductID($item['param2']);
                    $r -> setAction('SetCurrentProduct');
                    $r -> setKey('prod_id', $item['param2']);
                    $r -> setKey('category_id', $item['param1']);
                }
                $result = $r -> getURL();
                break;
        }

        return $result;
    }

    /**
     * Processes local tags
     */
    function getTag($tag)
    {
        return getKeyIgnoreCase($tag, $this->_Template_Contents);
    }

    var $NoView;
    var $mTmplFiller;
    var $_Template_Contents;
    var $_Menu_Data;
};

?>