<?php
/***********************************************************************
| Avactis (TM) Shopping Cart software developed by Pentasoft Corp.
| http://www.avactis.com
| -----------------------------------------------------------------------
| All source codes & content (c) Copyright 2004-2010, Pentasoft Corp.
| unless specifically noted otherwise.
| =============================================
| This source code is released under the Avactis License Agreement.
| The latest version of this license can be found here:
| http://www.avactis.com/license.php
|
| By using this software, you acknowledge having read this license agreement
| and agree to be bound thereby.
|
 ***********************************************************************/
?><?php

/**
 * @package PaymentModuleChaseCC
 * @author Egor V. Derevyankin
 */

class Payment_Module_Chase_CC extends pm_sm_api
{

    /* static */ function getInitialCurrencySettings()
    {
        $res = array
        (
            'currency_acceptance_rules' => array
            (
                array
                (
                    'rule_name'     =>  ACTIVE_AND_ACCEPTED_BY_PM_SM_AND_SELECTED_BY_CUSTOMER
                   ,'rule_selected' =>  DB_FALSE
                )
               ,array
                (
                    'rule_name'     =>  THE_ONLY_ACCEPTED
                   ,'rule_selected' =>  DB_TRUE
                )
               ,array
                (
                    'rule_name'     =>  MAIN_STORE_CURRENCY
                   ,'rule_selected' =>  DB_FALSE
                )
            )

            /**
             * Website Payments Standard Integration Guide, Last updated: March 2008.
             * TABLE 2.6 Currencies Allowed for Transactions and Balances, Currency ISO-4217 Code
             */
           ,'accepted_currencies' => array
            (
                array
                (
                    'currency_code'   => 'AUD'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'CAD'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'CHF'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'CZK'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'DKK'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'EUR'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'GBP'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'HKD'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'HUF'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'JPY'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'NOK'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'NZD'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'PLN'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'SEK'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'SGD'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'USD'
                   ,'currency_status' => 'THE_ONE_ONLY_ACCEPTED'
                )
            )
        );
        return $res;
    }

    function Payment_Module_Chase_CC()
    {}

    function install()
    {
        global $application;
        loadCoreFile('csv_parser.php');
        $csv_parser = new CSV_Parser();

        $tables = Payment_Module_Chase_CC::getTables();
        $query = new DB_Table_Create($tables);

        $table = 'pm_chase_settings';
        $columns = $tables[$table]['columns'];

        list($flt,$Default_Settings) = $csv_parser->parse_file(dirname(__FILE__)."/includes/default_settings.csv");
        if(count($Default_Settings) > 0)
        {
            foreach($Default_Settings as $key => $setting)
            {
                $query = new DB_Insert($table);
                $query->addInsertValue($setting["key"], $columns['setting_key']);
                $query->addInsertValue($setting["value"], $columns['setting_value']);
                $application->db->getDB_Result($query);
            };
        };

        //                                RemovePersonInfoType
        //                   ,          -     CheckoutEditor
        //             PersonInfoType,         , BillingInfo, ShippingInfo,
        //  CustomerInfo, CreditCardInfo, BankAccountInfo, PaymentMethodInfo (
        //             ), ShippingMethodInfo (              ).
        //       PaymentModule'                                                 ,
        //                                     .
        //                    ,                 PersonInfoType          .
        //
        //  PersonInfoType.
        modApiFunc('EventsManager',
               'addEventHandler',
               'RemovePersonInfoTypeEvent',
               'Payment_Module_Chase_CC',
               'OnRemovePersonInfoType');
    }

    /**
     *                    RemovePersonInfoType.
     *                  ,          -     CheckoutEditor
     *            PersonInfoType,         , BillingInfo, ShippingInfo,
     * CustomerInfo, CreditCardInfo, BankAccountInfo, PaymentMethodInfo (
     *            ), ShippingMethodInfo (              ).
     *      PaymentModule'                                                 ,
     *                                    .
     *                   ,                 PersonInfoType          .
     *
     * PersonInfoType.
     *
     *        Payment_Module_Chase_CC                    2 PersonInfoType' :
     * billingInfo
     * creditCardInfo
     */
    function OnRemovePersonInfoType($person_info_type_tag)
    {
        global $application;

        if($this->isActive() === false)
        {
            //                        -                                .
            $value =  NULL;
        }
        else
        {
            $this->MessageResources = &$application->getInstance('MessageResources',"payment-module-chase-messages", "AdminZone");
            $msg = $this->MessageResources->getMessage('MODULE_PAYMENT_CHASE_API_MSG_REMOVE_PERSON_INFO_TYPE');

            $value = NULL;
            switch($person_info_type_tag)
            {
                case "billingInfo":
                {
                    $value = $msg;
                    break;
                }
                case "creditCardInfo":
                {
                    $value = $msg;
                    break;
                }
                default:
                {
                    break;
                }
            }
        }
        return $value;
    }

    function uninstall()
    {
        $query = new DB_Table_Delete(Payment_Module_Chase_CC::getTables());
    }

    function getTables()
    {
        static $tables;

        if (is_array($tables))
        {
            return $tables;
        }

        $tables = array();

        $table = 'pm_chase_settings';
        $tables[$table] = array(
            'columns' => array(
                'setting_id'    => $table.'.setting_id'
               ,'setting_key'   => $table.'.setting_key'
               ,'setting_value' => $table.'.setting_value'
             )
           ,'types'   => array(
                'setting_id'    => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'setting_key'   => DBQUERY_FIELD_TYPE_CHAR255.' NOT NULL'
               ,'setting_value' => DBQUERY_FIELD_TYPE_CHAR255.' NOT NULL'
             )
           ,'primary' => array(
                'setting_id'
             )
        );

        global $application;
        return $application->addTablePrefix($tables);
    }

    function isActive()
    {
        global $application;
        $active_modules = modApiFunc("Checkout","getActiveModules","payment");
        return array_key_exists($this->getUid(), $active_modules);
    }

    function getUID()
    {
//        return 'F52817FB-BF07-414E-9BC8-B680EC03712E';
        include(dirname(__FILE__)."/includes/uid.php");
        return $uid;
    }

    function getInfo()
    {
        global $application;

        $MR = &$application->getInstance('MessageResources','payment-module-chase-messages','AdminZone');

        return array("GlobalUniquePaymentModuleID" => $this->getUid(),
                     "Name"        => $MR->getMessage('MODULE_NAME'),
                     "Description" => $MR->getMessage('MODULE_DESCR'),
                     "StatusMessage" => ($this->IsActive())? "<span class=\"status_online\">".$MR->getMessage('MODULE_STATUS_ACTIVE')."</span>":$MR->getMessage('MODULE_STATUS_INACTIVE')."<span class=\"required\">*</span>",
                     "PreferencesAZViewClassName" => "CheckoutPaymentModuleChaseCCInputAZ",
                     "CZInputViewClassName" => "CheckoutPaymentModuleChaseCCInput",
                     "CZOutputViewClassName" => "CheckoutPaymentModuleChaseCCOutput",
                     "APIClassName" => __CLASS__
                    );
    }

    function getSettings()
    {
        $res = execQuery('SELECT_PM_SM_SETTINGS2',array('ApiClassName' => __CLASS__, 'settings_table_name' => 'pm_chase_settings'));
        $settings=array();

        foreach($res as $k => $sval)
            $settings[$sval['setting_key']]=$sval['setting_value'];

        return $settings;
    }

    function updateSettings($settings)
    {
        global $application;
        $tables=$this->getTables();
        $stable=$tables['pm_chase_settings']['columns'];

        foreach($settings as $skey => $sval)
        {
            $query = new DB_Update('pm_chase_settings');
            $query->addUpdateValue($stable['setting_value'],$sval);
            $query->WhereValue($stable['setting_key'], DB_EQ, $skey);
            $application->db->PrepareSQL($query);
            $application->db->DB_Exec();
        };

        return;
    }

    function getPaymentMode()
    {
        return "direct";
    }

    function getAdditionalPersonInfoVariantTag($prerequisite_type)
    {
        if($prerequisite_type == "creditCardInfo")
        {
            return "default";
        }
        else
        {
            return false;
        }
    }

    function export_cc_month($month)
    {
    	return sprintf("%02d", (int)$month);
    }

    function export_cc_year($year)
    {
    	if(((int)$year) < 100)
    	{
    		$year = 2000 + (int)$year;
    	}
        return sprintf("%04d", ((int)$year));
    }

    function export_cc_year_solo($year)
    {
        return sprintf("%02d", ((int)$year) % 100);
    }

    function export_zip($zip)
    {
    	if(empty($zip))
    	{
    		return "";
    	}
    	else
    	{
    		if(_ml_strlen($zip) == 5)
    		{
    			return $zip;
    		}
    		elseif(_ml_strlen($zip) == 9)
    		{
    			//ZIP + 4.                .
    			return (_ml_substr($zip, 0, 5) . '-' . _ml_substr($zip, 5, 4));
    		}
    		else
    		{
    			//          ,
    			return $zip;
    		}
    	}
    }

    function export_country_code($cid)
    {
    	if(empty($cid))
    	{
    		return "";
    	}
    	else
    	{
    		/*
    		Cardholder Billing Address Country Code:
            Required if processing a U.K. based Address. Valid values:
            US   United States
            CA   Canada
            GB   Great Britain
            UK   United Kingdom
                  Blank for all other countries
            */
            $asc_code = modApiFunc("Location", "getCountryCode", $cid);
            switch($asc_code)
            {
            	case "US":
                {
            	    return "US";
                }
                case "CA":
                {
                    return "CA";
                }
                case "GB":
                {
                    return "GB";
                }
                //No case for United Kingdom
            	default:
                {
            		return " ";
            	}
            }
    	}
    }

    function backgroundProcess($type="direct", $transactionData = null)
    {
        global $application;
        loadCoreFile('aal.class.php');

        $settings = $this->getSettings();

        if($settings['MODULE_MODE'] == CHASE_MODE_TEST)
        {
            $chase = &$application->getInstance('ChasePayment', $settings['CHASE_MERCHANT_ID'], CHASE_TEST_GATEWAY_URL, CHASE_TEST_GATEWAY_URN, $settings['CHASE_BIN']);
        }
        else
        {
            $chase = &$application->getInstance('ChasePayment', $settings['CHASE_MERCHANT_ID'], CHASE_DEFAULT_GATEWAY_URL, CHASE_DEFAULT_GATEWAY_URN, $settings['CHASE_BIN']);
        };

        $billingInfo = new ArrayAccessLayer(modApiFunc("Checkout", "getPrerequisiteValidationResults", "billingInfo"));
        $billingInfo->setAccessMask("validatedData", AAL_CUSTOM_PARAM, "value");

        $CCInfo = new ArrayAccessLayer(modApiFunc("Checkout", "getDecryptedCreditCardInfoPrerequisiteData", $this->getUid()));
        $CCInfo->setAccessMask(AAL_CUSTOM_PARAM, "value");


        $chase->setCcAccountNum($CCInfo->getByMask('CreditCardNumber'));
        //- Formats: YYYYMM
        $chase->setCcExp($this->export_cc_year($CCInfo->getByMask('ExpirationYear')) .
                           $this->export_cc_month($CCInfo->getByMask('ExpirationMonth')));

        //           Visa     Discover                 CVV2       -          2 (2 - Value on card but illegible)
        //               .                                                .
        $cc_type = $CCInfo->getByMask('CreditCardType');
        $cc_cvv2 = $CCInfo->getByMask('CreditCardVerificationNumber');
        if(($cc_type == 'Visa' || $cc_type == 'Discover') &&
            empty($cc_cvv2))
        {
            $chase->setCcCardVerifyPresenceInd("2");
        }
        else
        {
            $chase->setCcCardVerifyPresenceInd("");
        }

        if(!empty($cc_cvv2))
        {
        	$chase->setCcCardVerifyNum($cc_cvv2);
        }
        else
        {
        	$chase->setCcCardVerifyNum("");
        }

        $cc_start_month = $CCInfo->getByMask('ValidFromMonth');
        $cc_start_year = $CCInfo->getByMask('ValidFromYear');

        // add support for Switch cards.
        //Switch is a former debit card in the United Kingdom.
        //It was a sister to the Solo debit card.
        //Switch was launched in 1988 by Midland Bank, National Westminster Bank and
        //  the Royal Bank of Scotland as a multifunction cheque guarantee and cash card.
        //The brand was merged with Maestro, an international debit card brand owned by
        //  MasterCard Worldwide, in 2002.
        // ... Consequently many retailers who advertise that they accept Maestro can only
        //  accept UK-issued cards, ie. former Switch cards.
        if($cc_type == 'Solo')
        {
        	if(!empty($cc_start_month))
        	{
        		$chase->setSwitchSoloCardStartDate($this->export_cc_month($cc_start_month) .
        		                                     $this->export_cc_year_solo($cc_start_year));
        	}
        	else
        	{
        		$chase->setSwitchSoloCardStartDate("");
        	}
        }
        else
        {
            $chase->setSwitchSoloCardStartDate("");
        }

        $chase->setAvsZip($this->export_zip($billingInfo->getByMask('Postcode')));
        $chase->setAvsAddress1($billingInfo->getByMask('Streetline1'));
        $chase->setAvsAddress2($billingInfo->getByMask('Streetline2'));
        $chase->setAvsCity($billingInfo->getByMask('City'));
        $chase->setAvsState($billingInfo->getByMask('Statemenu')? modApiFunc("Location", "getStateCode", $billingInfo->getByMask('Statemenu')): $billingInfo->getByMask('Statetext'));
        $chase->setAvsName($billingInfo->getByMask('Firstname')." ".$billingInfo->getByMask('Lastname'));
        $chase->setAvsCountryCode($this->export_country_code($billingInfo->getByMask('Country')));
        $chase->setOrderID(CHASE_ORDER_NOT_CREATED);
        $chase->setAmount($this->export_PRICE_N_A(modApiFunc("Checkout", "getOrderPrice", "Total", modApiFunc("Localization", "whichCurrencySendOrderToPaymentShippingGatewayIn", ORDER_NOT_CREATED_YET, $this->getUid()))));
        $chase->setComments(" ");

        $step_id_to_redirect_to = modApiFunc("Checkout", "getCurrentStepID");
        $res = $chase->doPayment();
        if(isset($res['Error']))
        {
        	//                                                       .
            modApiFunc('Session','set','Chase_ErrorMessage',$res['ErrorMessage']);
            $paymentModule = modApiFunc("Checkout", "getPrerequisiteValidationResults", "paymentModule");
            modApiFunc("Checkout", "setPrerequisitesValidationResultsItem", "paymentModule", "", false, "CHECKOUT_ERR_005", array(), $paymentModule["validatedData"]);
            $step_id_to_redirect_to = modApiFunc('Checkout', 'getStepIDtoRedirectToAfterPrerequisitesValidationErrors', modApiFunc("Checkout", "getCurrentStepID"));
            modApiFunc("Checkout", "saveState");
        }
        else
        {
        	//                                       .
        	//                                     "DECLINED"     "UNABLE".
            //Create order
            $order_id = modApiFunc("Checkout", "createOrderInDB");
            //                .
            //
            modApiFunc("Cart", "removeAllFromCart");

            $MR = &$application->getInstance('MessageResources','payment-module-chase-messages','AdminZone');
            $SMR = &$application->getInstance('MessageResources');

            $txn_status = "";
            $new_order_payment_status = ORDER_PAYMENT_STATUS_WAITING;

            //                       - Approved, Declined,
            //  (                              ).
            $new_order_payment_status = "unchanged";
            $new_order_payment_status_msg = "";
            $approvalStatusMsg = "";
            switch($res['APPROVALSTATUS'])
            {
                case CHASE_APPROVAL_STATUS_DECLINE:
                {
                    $new_order_payment_status = ORDER_PAYMENT_STATUS_DECLINED;
                    $new_order_payment_status_msg = $SMR->getMessage("ORDER_PAYMENT_STATUS_003");
                    $approvalStatusMsg = $MR->getMessage("MODULE_PAYMENT_CHASE_LOG_MESSAGE_APPROVAL_STATUS_DECLINE");
                    break;
                }
                case CHASE_APPROVAL_STATUS_APPROVED:
                {
                    $new_order_payment_status = ORDER_PAYMENT_STATUS_FULLY_PAID;
                    $new_order_payment_status_msg = $SMR->getMessage("ORDER_PAYMENT_STATUS_002");
                    $approvalStatusMsg = $MR->getMessage("MODULE_PAYMENT_CHASE_LOG_MESSAGE_APPROVAL_STATUS_APPROVED");
                    break;
                }
                default:
                {
                    $new_order_payment_status = "unchanged";
                    $approvalStatusMsg = $MR->getMessage("MODULE_PAYMENT_CHASE_LOG_MESSAGE_APPROVAL_STATUS_UNKNOWN");
                    break;
                }
            }

            $msg = $MR->getMessage('MODULE_PAYMENT_CHASE_LOG_MODULE_ID') . "\n" .
                $this->getUID() . "\n" .
                $this->getmicrotime() . "\n" .
                $settings["MODULE_NAME"] . " ".
                $MR->getMessage("MODULE_PAYMENT_CHASE_LOG_EVENT_DESCRIPTION") .
                $MR->getMessage("MODULE_PAYMENT_CHASE_LOG_MESSAGE_PASSED") . "\n" .

                $MR->getMessage("MODULE_PAYMENT_CHASE_LOG_MESSAGE_PROC_STATUS") . ':' . $res['PROCSTATUS'] . ':' . $res['PROCSTATUSMESSAGE'] . "\n" .
                $MR->getMessage("MODULE_PAYMENT_CHASE_LOG_MESSAGE_APPROVAL_STATUS") . ':' . $res['APPROVALSTATUS'] . ':' . $approvalStatusMsg . "\n" .

                $MR->getMessage("MODULE_PAYMENT_CHASE_LOG_MESSAGE_FULL_GATEWAY_RESPONSE") . ':' . $res['RawGatewayResponse'] . "\n";

            modApiFunc("Checkout", "addOrderHistory", $order_id, $msg);

            // update order payment status info
            if($new_order_payment_status != "unchanged")
            {
                $msg = $MR->getMessage("MODULE_PAYMENT_CHASE_LOG_MESSAGE_ORDER_PAYMENT_STATUS_UPDATED", array($new_order_payment_status_msg));
                modApiFunc("Checkout", "UpdatePaymentStatusInDB", $order_id, $new_order_payment_status, $msg);
            }

            $step_id_to_redirect_to = modApiFunc("Checkout", "getCurrentStepID") + 1;
        }

        $request = new Request();
        $request->setView('CheckoutView');
        $request->setAction("SetCurrStep");
        $request->setKey   ( 'step_id', $step_id_to_redirect_to);
        $application->redirect($request);
    }

    function pow10($exp)
    {
    	$res = 1;
    	for(;$exp >0;$exp--)
    	{
    		$res *= 10;
    	}
    	return $res;
    }

    function export_PRICE_N_A($price)
    {
        return ($price == PRICE_N_A) ? 0 : (int)round($price * $this->pow10(modApiFunc("Localization", "getValue", "CURRENCY_FORMAT_EXPONENT")));
    }

    function getmicrotime()
    {
        global $application;
        list($usec, $sec) = explode(" ", microtime());

        $MR = &$application->getInstance('MessageResources','payment-module-chase-messages','AdminZone');

        return $MR->getMessage('MODULE_PAYMENT_CHASE_LOG_EVENT_TIME') .
                (float)($usec + $sec) . " " .
                $MR->getMessage('MODULE_PAYMENT_CHASE_SECONDS');
    }
}
?>