<?php
/***********************************************************************
| Avactis (TM) Shopping Cart software developed by Pentasoft Corp.
| http://www.avactis.com
| -----------------------------------------------------------------------
| All source codes & content (c) Copyright 2004-2010, Pentasoft Corp.
| unless specifically noted otherwise.
| =============================================
| This source code is released under the Avactis License Agreement.
| The latest version of this license can be found here:
| http://www.avactis.com/license.php
|
| By using this software, you acknowledge having read this license agreement
| and agree to be bound thereby.
|
 ***********************************************************************/
?><?php
 /*
  * EwayPayment.php
  * Electronic Payment XML Interface for eWAY
  *
  * (c) Copyright Matthew Horoschun, CanPrint Communications 2005.
  *
  * $Id: EwayPayment.php,v 1.2 2005/04/18 03:30:33 matthew Exp $
  *
  * Date:    2005-04-18
  * Version: 2.0
  *
  */

class EwayPayment {
    var $parser;
    var $xmlData;
    var $currentTag;

    var $myGatewayURL;
    var $myCustomerID;

    var $myTotalAmount;
    var $myCustomerFirstname;
    var $myCustomerLastname;
    var $myCustomerEmail;
    var $myCustomerAddress;
    var $myCustomerPostcode;
    var $myCustomerInvoiceDescription;
    var $myCustomerInvoiceRef;
    var $myCardHoldersName;
    var $myCardNumber;
    var $myCardExpiryMonth;
    var $myCardExpiryYear;
    var $myTrxnNumber;
    var $myOption1;
    var $myOption2;
    var $myOption3;
    var $CVN;

    var $myResultTrxnStatus;
    var $myResultTrxnNumber;
    var $myResultTrxnOption1;
    var $myResultTrxnOption2;
    var $myResultTrxnOption3;
    var $myResultTrxnReference;
    var $myResultTrxnError;
    var $myResultAuthCode;
    var $myResultReturnAmount;

    var $myError;
    var $myErrorMessage;

    /***********************************************************************
     *** XML Parser - Callback functions                                 ***
     ***********************************************************************/
    function epXmlElementStart ($parser, $tag, $attributes) {
        $this->currentTag = $tag;
    }

    function epXmlElementEnd ($parser, $tag) {
        $this->currentTag = "";
    }

    function epXmlData ($parser, $cdata) {
        $this->xmlData[$this->currentTag] = $cdata;
    }

    /***********************************************************************
     *** SET values to send to eWAY                                      ***
     ***********************************************************************/
    function setCustomerID( $customerID ) {
        $this->myCustomerID = $customerID;
    }

    function setTotalAmount( $totalAmount ) {
        $this->myTotalAmount = $totalAmount;
    }

    function setCustomerFirstname( $customerFirstname ) {
        $this->myCustomerFirstname = $customerFirstname;
    }

    function setCustomerLastname( $customerLastname ) {
        $this->myCustomerLastname = $customerLastname;
    }

    function setCustomerEmail( $customerEmail ) {
        $this->myCustomerEmail = $customerEmail;
    }

    function setCustomerAddress( $customerAddress ) {
        $this->myCustomerAddress = $customerAddress;
    }

    function setCustomerPostcode( $customerPostcode ) {
        $this->myCustomerPostcode = $customerPostcode;
    }

    function setCustomerInvoiceDescription( $customerInvoiceDescription ) {
        $this->myCustomerInvoiceDescription = $customerInvoiceDescription;
    }

    function setCustomerInvoiceRef( $customerInvoiceRef ) {
        $this->myCustomerInvoiceRef = $customerInvoiceRef;
    }

    function setCardHoldersName( $cardHoldersName ) {
        $this->myCardHoldersName = $cardHoldersName;
    }

    function setCardNumber( $cardNumber ) {
        $this->myCardNumber = $cardNumber;
    }

    function setCardExpiryMonth( $cardExpiryMonth ) {
        $this->myCardExpiryMonth = $cardExpiryMonth;
    }

    function setCardExpiryYear( $cardExpiryYear ) {
        $this->myCardExpiryYear = $cardExpiryYear;
    }

    function setTrxnNumber( $trxnNumber ) {
        $this->myTrxnNumber = $trxnNumber;
    }

    function setOption1( $option1 ) {
        $this->myOption1 = $option1;
    }

    function setOption2( $option2 ) {
        $this->myOption2 = $option2;
    }

    function setOption3( $option3 ) {
        $this->myOption3 = $option3;
    }

    function setCVN( $CVN ) {
        $this->myCVN = $CVN;
    }

    /***********************************************************************
     *** GET values returned by eWAY                                     ***
     ***********************************************************************/
    function getTrxnStatus() {
        return $this->myResultTrxnStatus;
    }

    function getTrxnNumber() {
        return $this->myResultTrxnNumber;
    }

    function getTrxnOption1() {
        return $this->myResultTrxnOption1;
    }

    function getTrxnOption2() {
        return $this->myResultTrxnOption2;
    }

    function getTrxnOption3() {
        return $this->myResultTrxnOption3;
    }

    function getTrxnReference() {
        return $this->myResultTrxnReference;
    }

    function getTrxnError() {
        return $this->myResultTrxnError;
    }

    function getAuthCode() {
        return $this->myResultAuthCode;
    }

    function getReturnAmount() {
        return $this->myResultReturnAmount;
    }

    function getCVN() {
        return $this->myCVN;
    }

    function getError()
    {
        if( $this->myError != 0 ) {
            // Internal Error
            return $this->myError;
        } else {
            // eWAY Error
            if( $this->getTrxnStatus() == 'True' ) {
                return EWAY_TRANSACTION_OK;
            } elseif( $this->getTrxnStatus() == 'False' ) {
                return EWAY_TRANSACTION_FAILED;
            } else {
                return EWAY_TRANSACTION_UNKNOWN;
            }
        }
    }

    function getErrorMessage()
    {
        if( $this->myError != 0 ) {
            // Internal Error
            return $this->myErrorMessage;
        } else {
            // eWAY Error
            return $this->getTrxnError();
        }
    }

    /***********************************************************************
     *** Class Constructor                                               ***
     ***********************************************************************/
    function EwayPayment( $customerID = EWAY_DEFAULT_CUSTOMER_ID, $gatewayURL = EWAY_DEFAULT_GATEWAY_URL ) {
        $this->myCustomerID = $customerID;
        $this->myGatewayURL = $gatewayURL;
    }

    /***********************************************************************
     *** Business Logic                                                  ***
     ***********************************************************************/
    function doPayment() {
        $xmlRequest = "<ewaygateway>".
                "<ewayCustomerID>"._ml_htmlentities( $this->myCustomerID )."</ewayCustomerID>".
                "<ewayTotalAmount>"._ml_htmlentities( $this->myTotalAmount)."</ewayTotalAmount>".
                "<ewayCustomerFirstName>"._ml_htmlentities( $this->myCustomerFirstname )."</ewayCustomerFirstName>".
                "<ewayCustomerLastName>"._ml_htmlentities( $this->myCustomerLastname )."</ewayCustomerLastName>".
                "<ewayCustomerEmail>"._ml_htmlentities( $this->myCustomerEmail )."</ewayCustomerEmail>".
                "<ewayCustomerAddress>"._ml_htmlentities( $this->myCustomerAddress )."</ewayCustomerAddress>".
                "<ewayCustomerPostcode>"._ml_htmlentities( $this->myCustomerPostcode )."</ewayCustomerPostcode>".
                "<ewayCustomerInvoiceDescription>"._ml_htmlentities( $this->myCustomerInvoiceDescription )."</ewayCustomerInvoiceDescription>".
                "<ewayCustomerInvoiceRef>"._ml_htmlentities( $this->myCustomerInvoiceRef )."</ewayCustomerInvoiceRef>".
                "<ewayCardHoldersName>"._ml_htmlentities( $this->myCardHoldersName )."</ewayCardHoldersName>".
                "<ewayCardNumber>"._ml_htmlentities( $this->myCardNumber )."</ewayCardNumber>".
                "<ewayCardExpiryMonth>"._ml_htmlentities( $this->myCardExpiryMonth )."</ewayCardExpiryMonth>".
                "<ewayCardExpiryYear>"._ml_htmlentities( $this->myCardExpiryYear )."</ewayCardExpiryYear>".
                "<ewayTrxnNumber>"._ml_htmlentities( $this->myTrxnNumber )."</ewayTrxnNumber>".
                "<ewayCVN>"._ml_htmlentities( $this->myCVN )."</ewayCVN>".
                "<ewayOption1>"._ml_htmlentities( $this->myOption1 )."</ewayOption1>".
                "<ewayOption2>"._ml_htmlentities( $this->myOption2 )."</ewayOption2>".
                "<ewayOption3>"._ml_htmlentities( $this->myOption3 )."</ewayOption3>".
                "</ewaygateway>";

        /* Use CURL to execute XML POST and write output into a string */
        /*
        $ch = curl_init( $this->myGatewayURL );
        curl_setopt( $ch, CURLOPT_POST, 1 );
        curl_setopt( $ch, CURLOPT_POSTFIELDS, $xmlRequest );
        curl_setopt( $ch, CURLOPT_RETURNTRANSFER, 1 );
        curl_setopt( $ch, CURLOPT_TIMEOUT, 240 );
        */

        loadCoreFile('bouncer.php');
        $bnc = new Bouncer();
        $bnc->setURL($this->myGatewayURL);
        $bnc->setMethod('POST');
        $bnc->setPOSTstring($xmlRequest);

        $result = $bnc->RunRequest();

        // Check whether the curl_exec worked.
        if( $result !== false) {
            $xmlResponse = $result["body"];

            // It worked, so setup an XML parser for the result.
            $this->parser = xml_parser_create();

            // Disable XML tag capitalisation (Case Folding)
            xml_parser_set_option ($this->parser, XML_OPTION_CASE_FOLDING, FALSE);

            // Define Callback functions for XML Parsing
            xml_set_object($this->parser, $this);
            xml_set_element_handler ($this->parser, "epXmlElementStart", "epXmlElementEnd");
            xml_set_character_data_handler ($this->parser, "epXmlData");

            // Parse the XML response
            xml_parse($this->parser, $xmlResponse, TRUE);

            if( xml_get_error_code( $this->parser ) == XML_ERROR_NONE ) {
                // Get the result into local variables.
                $this->myResultTrxnStatus = @$this->xmlData['ewayTrxnStatus'];
                $this->myResultTrxnNumber = @$this->xmlData['ewayTrxnNumber'];
                $this->myResultTrxnOption1 = @$this->xmlData['ewayTrxnOption1'];
                $this->myResultTrxnOption2 = @$this->xmlData['ewayTrxnOption2'];
                $this->myResultTrxnOption3 = @$this->xmlData['ewayTrxnOption3'];
                $this->myResultTrxnReference = @$this->xmlData['ewayTrxnReference'];
                $this->myResultAuthCode = @$this->xmlData['ewayAuthCode'];
                $this->myResultReturnAmount = @$this->xmlData['ewayReturnAmount'];
                $this->myResultTrxnError = @$this->xmlData['ewayTrxnError'];
                $this->myError = 0;
                $this->myErrorMessage = '';
            } else {
                // An XML error occured. Return the error message and number.
                $this->myError = xml_get_error_code( $this->parser ) + EWAY_XML_ERROR_OFFSET;
                $this->myErrorMessage = xml_error_string( @$myError );
            }
            // Clean up our XML parser
            xml_parser_free( $this->parser );
        } else {

            $this->myError = $bnc->errno;
            $this->myErrorMessage = $bnc->errstr;
        }
        return $this->getError();
    }
}
?>