<?php
/***********************************************************************
| Avactis (TM) Shopping Cart software developed by Pentasoft Corp.
| http://www.avactis.com
| -----------------------------------------------------------------------
| All source codes & content (c) Copyright 2004-2010, Pentasoft Corp.
| unless specifically noted otherwise.
| =============================================
| This source code is released under the Avactis License Agreement.
| The latest version of this license can be found here:
| http://www.avactis.com/license.php
|
| By using this software, you acknowledge having read this license agreement
| and agree to be bound thereby.
|
 ***********************************************************************/
?><?php

/**
 *
 * @package PaymentModuleMerchantPlusCC
 * @author Alexander Girin
 */
class Payment_Module_MerchantPlus_CC extends pm_sm_api
{
//------------------------------------------------
//               PUBLIC DECLARATION
//------------------------------------------------

    /**#@+
     * @access public
     */

    /* static */ function getInitialCurrencySettings()
    {
        $res = array
        (
            'currency_acceptance_rules' => array
            (
                array
                (
                    'rule_name'     =>  ACTIVE_AND_ACCEPTED_BY_PM_SM_AND_SELECTED_BY_CUSTOMER
                   ,'rule_selected' =>  DB_FALSE
                )
               ,array
                (
                    'rule_name'     =>  THE_ONLY_ACCEPTED
                   ,'rule_selected' =>  DB_TRUE
                )
               ,array
                (
                    'rule_name'     =>  MAIN_STORE_CURRENCY
                   ,'rule_selected' =>  DB_FALSE
                )
            )

            /**
             * Website Payments Standard Integration Guide, Last updated: March 2008.
             * TABLE 2.6 Currencies Allowed for Transactions and Balances, Currency ISO-4217 Code
             */
           ,'accepted_currencies' => array
            (
                array
                (
                    'currency_code'   => 'AUD'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'CAD'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'CHF'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'CZK'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'DKK'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'EUR'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'GBP'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'HKD'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'HUF'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'JPY'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'NOK'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'NZD'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'PLN'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'SEK'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'SGD'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'USD'
                   ,'currency_status' => 'THE_ONE_ONLY_ACCEPTED'
                )
            )
        );
        return $res;
    }

    /**
     * PaymentModuleMerchantPlus constructor.
     */
    function Payment_Module_MerchantPlus_CC()
    {
        global $application;
        $this->MessageResources = &$application->getInstance('MessageResources',"payment-module-merchantplus-messages", "AdminZone");
        //: gen real UID
        $info = $this->getInfo();

        $this->getSettings();

        $this->OrderHistoryMessageTag = $this->MessageResources->getMessage('MODULE_PAYMENT_MERCHANTPLUS_LOG_MODULE_ID') . $info["GlobalUniquePaymentModuleID"] . "\n" .
        $this->Settings["MODULE_NAME"];
        $this->errorMessage = null;
    }

    function loadState()
    {
        if(modApiFunc('Session', 'is_Set', 'MerchantPlusErrorMessage'))
        {
            $this->errorMessage = modApiFunc('Session', 'get', 'MerchantPlusErrorMessage');
        }
        else
        {
            $this->errorMessage = null;
        }
    }

    function saveState()
    {
        if ($this->errorMessage)
        {
            modApiFunc('Session', 'set', 'MerchantPlusErrorMessage', $this->errorMessage);
        }
    }

    function setErrorMessage($msg)
    {
        $this->errorMessage = $msg;
    }

    function getErrorMessage()
    {
        return $this->errorMessage;
    }

    function unsetErrorMessage()
    {
        $this->errorMessage = null;
        if(modApiFunc('Session', 'is_Set', 'MerchantPlusErrorMessage'))
        {
            modApiFunc('Session', 'un_Set', 'MerchantPlusErrorMessage');
        }
    }


    function getmicrotime()
    {
        global $application;

        list($usec, $sec) = explode(" ",microtime());

        return $this->MessageResources->getMessage('MODULE_PAYMENT_MERCHANTPLUS_LOG_EVENT_TIME') . (float)($usec + $sec) . " " . $this->MessageResources->getMessage('MODULE_PAYMENT_MERCHANTPLUS_SECONDS');
    }

    /**
     * Installs the specified module in the system.
     *
     * The install() method is called statically.
     * To call other methods of this class from this method,
     * the static call is used, for example,
     * Payment_Module_MerchantPlus_CC::getTables() instead of $this->getTables().
     */
    function install()
    {
        _use(dirname(__FILE__).'/includes/install.inc');

        //                                RemovePersonInfoType
        //                   ,          -     CheckoutEditor
        //             PersonInfoType,         , BillingInfo, ShippingInfo,
        //  CustomerInfo, CreditCardInfo, BankAccountInfo, PaymentMethodInfo (
        //             ), ShippingMethodInfo (              ).
        //       PaymentModule'                                                 ,
        //                                     .
        //                    ,                 PersonInfoType          .
        //
        //  PersonInfoType.
        modApiFunc('EventsManager',
               'addEventHandler',
               'RemovePersonInfoTypeEvent',
               'Payment_Module_MerchantPlus_CC',
               'OnRemovePersonInfoType');
    }

    /**
     *                    RemovePersonInfoType.
     *                  ,          -     CheckoutEditor
     *            PersonInfoType,         , BillingInfo, ShippingInfo,
     * CustomerInfo, CreditCardInfo, BankAccountInfo, PaymentMethodInfo (
     *            ), ShippingMethodInfo (              ).
     *      PaymentModule'                                                 ,
     *                                    .
     *                   ,                 PersonInfoType          .
     *
     * PersonInfoType.
     *
     *        Payment_Module_MerchantPlus_CC                        PersonInfoType' :
     * billingInfo
     * shippingInfo
     * creditCardInfo
     */
    function OnRemovePersonInfoType($person_info_type_tag)
    {
        global $application;

        if($this->isActive() === false)
        {
            //                        -                                .
            $value =  NULL;
        }
        else
        {
            $this->MessageResources = &$application->getInstance('MessageResources',"payment-module-merchantplus-messages", "AdminZone");
            $msg = $this->MessageResources->getMessage('MODULE_PAYMENT_MERCHANTPLUS_API_MSG_REMOVE_PERSON_INFO_TYPE');

            $value = NULL;
            switch($person_info_type_tag)
            {
                case "billingInfo":
                {
                    $value = $msg;
                    break;
                }
                case "shippingInfo":
                {
                    $value = $msg;
                    break;
                }
                case "creditCardInfo":
                {
                    $value = $msg;
                    break;
                }
                default:
                {
                    break;
                }
            }
        }
        return $value;
    }



    /**
     * This method is invoked by Checkout module in CZ (Customer Zone). It is
     * also invoked in AZ (Admin Zone).
     * returns - Boolean - Show this module during the process of Checkout in
     * Customer Zone or not?
     */
    function isActive()
    {
        global $application;
        $active_modules = modApiFunc("Checkout","getActiveModules","payment");
        return array_key_exists($this->getUid(), $active_modules);
    }

    function getUid()
    {
//        return "C5EC6759-0311-4A71-8537-49273508078F";
        include(dirname(__FILE__)."/includes/uid.php");
        return $uid;
    }

    /**
     * Gets module info.
     */
    function getInfo()
    {
        global $application;

        $this->getSettings();

        return array("GlobalUniquePaymentModuleID" => $this->getUid(),
                     "Name"        => $this->Settings["MODULE_NAME"],
//                     "Description" => $this->Settings["MODULE_DESCR"],
                     "StatusMessage" => ($this->IsActive())? "<span class=\"status_online\">".$this->MessageResources->getMessage('MODULE_STATUS_ACTIVE')."</span>":$this->MessageResources->getMessage('MODULE_STATUS_INACTIVE')."<span class=\"required\">*</span>",
                     //"Undefined", //NOT for active/inactive.
                     // there is special config in Checkout module to handle/store
                     // active/inactive flags.
                     "PreferencesAZViewClassName" => "merchantplus_cc_input_az",
                     "CZInputViewClassName" => "CheckoutPaymentModuleMerchantPlusCCInput",
                     "CZOutputViewClassName" => "CheckoutPaymentModuleMerchantPlusCCOutput",
                     "APIClassName" => __CLASS__
                    );
    }
    /**
     * Clears the Settings table.
     */
    function clearSettingsInDB()
    {
        global $application;
        $query = new DB_Delete('pm_merchantplus_settings_cc');
        $application->db->getDB_Result($query);
    }

    /**
     * Gets current module settings from Settings.
     *
     * @return array - module settings array
     */
    function getSettings()
    {
        $result = execQuery('SELECT_PM_SM_SETTINGS',array('ApiClassName' => __CLASS__, 'settings_table_name' => 'pm_merchantplus_settings_cc'));
        $this->Settings = array();
        for ($i=0; $i<sizeof($result); $i++)
        {
            $this->Settings[$result[$i]['set_key']] = unserialize($result[$i]['set_value']);
        }
        return $this->Settings;
    }

    /**
     * Sets up module attributes and logs it to the database.
     *
     * @param array $Settings - module settings array.
     */
    function setSettings($Settings)
    {
        global $application;
        $this->clearSettingsInDB();
        $tables = $this->getTables();
        $columns = $tables['pm_merchantplus_settings_cc']['columns'];

        foreach($Settings as $key => $value)
        {
            $query = new DB_Insert('pm_merchantplus_settings_cc');
            $query->addInsertValue($key, $columns['key']);
            $query->addInsertValue(serialize($value), $columns['value']);
            $application->db->getDB_Result($query);

            $inserted_id = $application->db->DB_Insert_Id();
        }
    }

    /**
     * Sets up module attributes and logs it to the database.
     *
     * @param array $Settings -  module settings array. The undefined parameter values
     * remain unchanged.
     */
    function updateSettings($Settings)
    {
        global $application;
        $tables = $this->getTables();
        $columns = $tables['pm_merchantplus_settings_cc']['columns'];

        foreach($Settings as $key => $value)
        {
            $query = new DB_Update('pm_merchantplus_settings_cc');
            $query->addUpdateValue($columns['value'], serialize($value));
            $query->WhereValue($columns['key'], DB_EQ, $key);
            $application->db->getDB_Result($query);
        }
    }

    /**
     * It is called from Checkout. It returns the cost of selected delivery
     * method. The cost should be 0 for the majority of payment methods.
     */
    function getPaymentCost($method_id)
    {
        return 0;
    }

    /**
     * Converts the value of the monetary sum to be used _out_ ASC.
     * If the price equals PRICE_N_A, then it is changed to 0.0.
     */
    function export_PRICE_N_A($price)
    {
        return ($price == PRICE_N_A) ? 0.0 : $price;
    }

    /**
     * Returns the 'derect' or 'redirect' payment method.
     */
    function getPaymentMode()
    {
        return "direct";
    }

    function getAdditionalPersonInfoVariantTag($prerequisite_type)
    {
        //           -                     PersonInfo Variant'    -         /        ...
        if($prerequisite_type == "creditCardInfo")
        {
            return "default";
        }
        else
        {
            return false;
        }
    }

    function backgroundProcess()
    {
        global $application;
        loadCoreFile('aal.class.php');

        $currency_id = modApiFunc("Localization", "whichCurrencySendOrderToPaymentShippingGatewayIn", ORDER_NOT_CREATED_YET, $this->getUid());
        $currency = modApiFunc("Localization", "getCurrencyCodeById", $currency_id);

        $billingInfo = new ArrayAccessLayer(modApiFunc("Checkout", "getPrerequisiteValidationResults", "billingInfo"));
        $billingInfo->setAccessMask("validatedData", AAL_CUSTOM_PARAM, "value");

        $shippingInfo = new ArrayAccessLayer(modApiFunc("Checkout", "getPrerequisiteValidationResults", "shippingInfo"));
        $shippingInfo->setAccessMask("validatedData", AAL_CUSTOM_PARAM, "value");

        $CCInfo = modApiFunc("Checkout", "getDecryptedCreditCardInfoPrerequisiteData", $this->getUid());
        //:                 $CCInfo             false.                                          .

        $Products = modApiFunc("Cart", "getCartContent", $currency_id);

        $DataFields = array
        (
           # Merchant Account Information
            "x_login"               => $this->Settings["MODULE_LOGIN"]
           ,"x_version"             => "3.1"
           ,"x_test_request"    => $this->Settings["MODULE_MODE"]=="1"? "TRUE":"FALSE" //Only for test purposes
           ,"x_tran_key"            => $this->Settings["MODULE_KEY"]
           ,"x_delim_char"          => $application->getAppIni("MERCHANTPLUS_DELIM_CHAR")
           ,"x_delim_data"          => "TRUE"
           ,"x_encap_char"          => $application->getAppIni("MERCHANTPLUS_ENCAP_CHAR")
           #Relay Response Fields
           ,"x_url"                 => "FALSE"
           ,"x_relay_response"      => "FALSE"
           #CC Info
           ,"x_card_num"            => $CCInfo["CreditCardNumber"]["value"]
           ,"x_exp_date"            => $CCInfo["ExpirationMonth"]["value"].$CCInfo["ExpirationYear"]["value"]
           ,"x_card_code"           => $CCInfo["CreditCardVerificationNumber"]["value"] ? $CCInfo["CreditCardVerificationNumber"]["value"] : ""
           ,"x_description"         => modApiFunc("Configuration", "getValue", SYSCONFIG_STORE_OWNER_NAME)
           #Customer Name and Billing Address
           ,"x_first_name"      => $billingInfo->getByMask('Firstname')
           ,"x_last_name"       => $billingInfo->getByMask('Lastname')
           ,"x_company"         => "" //Not defined as Billing info attribute
           ,"x_address"         => $billingInfo->getByMask('Streetline1')." ".$billingInfo->getByMask('Streetline2')
           ,"x_city"            => $billingInfo->getByMask('City')
           ,"x_state"           => $billingInfo->getByMask('Statemenu')? modApiFunc("Location", "getStateCode", $billingInfo->getByMask('Statemenu')): $billingInfo->getByMask('Statetext')
           ,"x_zip"             => $billingInfo->getByMask('Postcode')
           ,"x_country"         => $billingInfo->getByMask('Country')? modApiFunc("Location", "getCountryCode", $billingInfo->getByMask('Country')):""
           ,"x_phone"           => $billingInfo->getByMask('Phone')
           ,"x_fax"             => "" //Not defined as Billing info attribute
           #Customer Shipping Address
           ,"x_ship_to_first_name"  => $shippingInfo->getByMask('Firstname')
           ,"x_ship_to_last_name"   => $shippingInfo->getByMask('Lastname')
           ,"x_ship_to_company"     => "" //Not defined as Shipping info attribute
           ,"x_ship_to_address"     => $shippingInfo->getByMask('Streetline1')." ".$shippingInfo->getByMask('Streetline2')
           ,"x_ship_to_city"        => $shippingInfo->getByMask('City')
           ,"x_ship_to_state"       => $shippingInfo->getByMask('Statemenu')? modApiFunc("Location", "getStateCode", $shippingInfo->getByMask('Statemenu')): $shippingInfo->getByMask('Statetext')
           ,"x_ship_to_zip"         => $shippingInfo->getByMask('Postcode')
           ,"x_ship_to_country"     => $shippingInfo->getByMask('Country')? modApiFunc("Location", "getCountryCode", $shippingInfo->getByMask('Country')):""
           #Transaction Data
           ,"x_amount"              => $this->export_PRICE_N_A(modApiFunc("Checkout", "getOrderPrice", "Total", $currency_id))
           ,"x_currency_code"   => $currency
           ,"x_method"              => "CC"
           ,"x_type"                => "AUTH_CAPTURE"
//           ,"CZSESSID"          => session_id() //Client session id
        );

        if ($this->Settings['MODULE_CART'] == 1)
        {
            $x_line_items = array();
            $n = 1;
            foreach ($Products as $productInfo)
            {
                $x_line_items["x_line_item[".($n-1)."]"] = "item".($n++)."<|>".prepareHTMLDisplay($productInfo["Name"])."<|>".(isset($productInfo["ShortDescription"])? prepareHTMLDisplay(((_ml_strlen($productInfo["ShortDescription"]) >= 255)? (_ml_substr($productInfo["ShortDescription"], 0, 252)."..."):$productInfo["ShortDescription"])):"")."<|>".$productInfo["Quantity_In_Cart"]."<|>".$productInfo["SalePrice"]."<|>Y";
            }
            $DataFields = array_merge($DataFields, $x_line_items);
        }

        $fields = "";
        foreach($DataFields as $key => $value )
        {
            $fields .= $key."=".urlencode($value)."&";
        }

        loadCoreFile('bouncer.php');
        $bnc = new Bouncer();
        $bnc->setMethod('POST');
        $bnc->setURL('https://'.$this->Settings["MODULE_URL"]);
        $bnc->setPOSTstring(rtrim( $fields, "&"));
        $result = $bnc->RunRequest();

        $DataFields = array(
                            "x_response_code" => ""
                           ,"x_response_subcode" => ""
                           ,"x_response_reason_code" => ""
                           ,"x_response_reason_text" => ""
                           ,"x_auth_code" => ""
                           ,"x_avs_code" => ""
                           ,"x_trans_id" => ""
                           ,"x_invoice_num" => ""
                           ,"x_description" => ""
                           ,"x_amount" => ""
                           ,"x_method" => ""
                           ,"x_type" => ""
                           ,"x_cust_id" => ""
                           ,"x_first_name" => ""
                           ,"x_last_name" => ""
                           ,"x_company" => ""
                           ,"x_address" => ""
                           ,"x_city" => ""
                           ,"x_state" => ""
                           ,"x_zip" => ""
                           ,"x_country" => ""
                           ,"x_phone" => ""
                           ,"x_fax" => ""
                           ,"x_email" => ""
                           ,"x_ship_to_first_name" => ""
                           ,"x_ship_to_last_name" => ""
                           ,"x_ship_to_company" => ""
                           ,"x_ship_to_address" => ""
                           ,"x_ship_to_city" => ""
                           ,"x_ship_to_state" => ""
                           ,"x_ship_to_zip" => ""
                           ,"x_ship_to_country" => ""
                           ,"x_tax" => ""
                           ,"x_duty" => ""
                           ,"x_freight" => ""
                           ,"x_tax_exempt" => ""
                           ,"x_po_num" => ""
                           ,"x_md5_hash" => ""
                           ,"x_cvv2_resp_code" => ""
                           );

        $responseDataFields = array();
        if (isset($result["body"]) && $result["body"])
        {
            $responseDataFields = explode($application->getAppIni("MERCHANTPLUS_DELIM_CHAR"), $result["body"]);
        }

        $i = 0;
        foreach ($DataFields as $key => $val)
        {
            $DataFields[$key] = isset($responseDataFields[$i])? trim($responseDataFields[$i], $application->getAppIni("MERCHANTPLUS_ENCAP_CHAR")):"";
            $i++;
        }
//        $DataFields["CZSESSID"] = $responseDataFields[69];
        if ($DataFields["x_response_code"] == '1' || $DataFields["x_response_code"] == '4')
        {
            //Create order
            $order_id = modApiFunc("Checkout", "createOrderInDB");
            //                .
            //
            modApiFunc("Cart", "removeAllFromCart");
            $this->processData(array("_POST" => $DataFields, "_GET" => array()), $order_id);
            $step_id_to_redirect_to = modApiFunc("Checkout", "getCurrentStepID") + 1;
        }
        else
        {
            if (isset($DataFields["x_response_reason_code"]))
            {
                $this->setErrorMessage($this->MessageResources->getMessage(sprintf('RESPONSE_REASON_%03d', $DataFields["x_response_reason_code"])));
            }
            else
            {
                $this->setErrorMessage($bnc->getLastErrorMessage());
            }
            $paymentModule = modApiFunc("Checkout", "getPrerequisiteValidationResults", "paymentModule");
            modApiFunc("Checkout", "setPrerequisitesValidationResultsItem", "paymentModule", "", false, "CHECKOUT_ERR_005", array(), $paymentModule["validatedData"]);
            $step_id_to_redirect_to = modApiFunc('Checkout', 'getStepIDtoRedirectToAfterPrerequisitesValidationErrors', modApiFunc("Checkout", "getCurrentStepID"));
            modApiFunc("Checkout", "saveState");

        }
        $request = new Request();
        $request->setView('CheckoutView');
        $request->setAction("SetCurrStep");
        $request->setKey   ( 'step_id', $step_id_to_redirect_to);
        $application->redirect($request);
    }

    /**
     * Checks md5 hash sent from the gateway.
     */
    function checkMD5Hash($order_id, $hash, $transaction_id, $amount, $result)
    {
        if (_ml_strtoupper($hash) != _ml_strtoupper(md5($this->Settings['MODULE_HASH_VALUE'].$this->Settings['MODULE_LOGIN'].$transaction_id.$amount)))
        {
            $msg = $this->OrderHistoryMessageTag . "\n" . $this->getmicrotime() . "\n" . $this->MessageResources->getMessage('MODULE_PAYMENT_MERCHANTPLUS_LOG_EVENT_DESCRIPTION') . $this->MessageResources->getMessage('TRANSACTION_HASH_INCORRECT');
            $result = modApiFunc("Checkout", "UpdatePaymentStatusInDB", $order_id, 3, $msg);
        }
        return $result;
    }

    /**
     * Processes data on updating the order status, come from the payment gateway.
     */
    function processData($data, $order_id)
    {
        global $application;

        foreach ($data["_POST"] as $key => $val)
        {
            unset($data["_POST"][$key]);
            $data["_POST"][_ml_strtolower($key)] = $val;
        }
        $status = $data["_POST"]["x_response_code"];

        //Add the transaction number to the order
        modApiFunc("Checkout", "updateOrderPaymentProcessorOrderID", $order_id, $data["_POST"]["x_trans_id"]);
        $msg = $this->OrderHistoryMessageTag . "\n" . $this->getmicrotime() . "\n" . $this->MessageResources->getMessage('MODULE_PAYMENT_MERCHANTPLUS_LOG_EVENT_DESCRIPTION') . $this->MessageResources->getMessage(sprintf('RESPONSE_REASON_%03d', $data["_POST"]["x_response_reason_code"])) ."\n" . $this->MessageResources->getMessage('MODULE_PAYMENT_MERCHANTPLUS_LOG_POST_DATA') . " \n " . print_r($data["_POST"], true);
        switch ($status)
        {
            case '1':
                $EventType = "ConfirmationSuccess";
                $result = modApiFunc("Checkout", "UpdatePaymentStatusInDB", $order_id, 2, $msg);
                break;
            case '2':
            case '3':
                $EventType = "ConfirmationFailure";
                $result = modApiFunc("Checkout", "UpdatePaymentStatusInDB", $order_id, 3, $msg);
                break;
            case '4':
                $EventType = "ConfirmationSuccess";
                $result = modApiFunc("Checkout", "UpdatePaymentStatusInDB", $order_id, 1, $msg);
                break;
            default:
                //: Report Error: e.g. write to order history
                $result = array("payment_status" => array());
                break;
        }
//        $result = $this->checkMD5Hash($order_id, $data["_POST"]["x_md5_hash"], $data["_POST"]["x_trans_id"], $data["_POST"]["x_amount"], $result);
        return array("EventType" => $EventType, "statusChanged" => $result);
    }

    /**
     * Uninstalls the module.
     * It deletes all module tables.
     *
     * The uninstall() method is called statically.
     * To call other methods of this class from this method,
     * the static call is used, for example,
     * Payment_Module_Paypal_CC::getTables() instead of $this->getTables()
     */
    function uninstall()
    {
        $query = new DB_Table_Delete(Payment_Module_MerchantPlus_CC::getTables());
        global $application;
        $application->db->getDB_Result($query);
    }

    /**
     * Gets the array of meta description of module tables.
     *
     * The array structure of meta description of the table:
     * <code>
     *      $tables = array ();
     *      $table_name = 'table_name';
     *      $tables[$table_name] = array();
     *      $tables[$table_name]['columns'] = array
     *      (
     *          'fn1'               => 'table_name.field_name_1'
     *         ,'fn2'               => 'table_name.field_name_2'
     *         ,'fn3'               => 'table_name.field_name_3'
     *         ,'fn4'               => 'table_name.field_name_4'
     *      );
     *      $tables[$table_name]['types'] = array
     *      (
     *          'fn1'               => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
     *         ,'fn2'               => DBQUERY_FIELD_TYPE_INT .' NOT NULL'
     *         ,'fn3'               => DBQUERY_FIELD_TYPE_CHAR255
     *         ,'fn4'               => DBQUERY_FIELD_TYPE_TEXT
     *      );
     *      $tables[$table_name]['primary'] = array
     *      (
     *          'fn1'       # several key fields may be used, e.g. - 'fn1', 'fn2'
     *      );
     *      $tables[$table_name]['indexes'] = array
     *      (
     *          'index_name1' => 'fn2'      # several fields can be used in one index, e.g. - 'fn2, fn3'
     *         ,'index_name2' => 'fn3'
     *      );
     * </code>
     *
     * @return array -  the meta description of module tables
     */
    function getTables()
    {
        static $tables;

        if (is_array($tables))
        {
            return $tables;
        }

        $tables = array ();

        $settings = 'pm_merchantplus_settings_cc';
        $tables[$settings] = array();
        $tables[$settings]['columns'] = array
            (
                'id'                => $settings.'.pm_merchantplus_setting_id'
               ,'key'               => $settings.'.pm_merchantplus_setting_key'
               ,'value'             => $settings.'.pm_merchantplus_setting_value'
            );
        $tables[$settings]['types'] = array
            (
                'id'                => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'key'               => DBQUERY_FIELD_TYPE_CHAR50
               ,'value'             => DBQUERY_FIELD_TYPE_CHAR255
            );
        $tables[$settings]['primary'] = array
            (
                'id'
            );

        global $application;
        return $application->addTablePrefix($tables);
    }

    /**#@-*/

//------------------------------------------------
//              PRIVATE DECLARATION
//------------------------------------------------

    /**#@+
     * @access private
     */

    /**
     * A flag, which is stored in the database. It indicates, if the specified
     * module is mapped in CZ Checkout.
     */
    var $bActive;

    var $OrderHistoryMessageTag;
    /**#@-*/

}
?>