<?php
/***********************************************************************
| Avactis (TM) Shopping Cart software developed by Pentasoft Corp.
| http://www.avactis.com
| -----------------------------------------------------------------------
| All source codes & content (c) Copyright 2004-2010, Pentasoft Corp.
| unless specifically noted otherwise.
| =============================================
| This source code is released under the Avactis License Agreement.
| The latest version of this license can be found here:
| http://www.avactis.com/license.php
|
| By using this software, you acknowledge having read this license agreement
| and agree to be bound thereby.
|
 ***********************************************************************/
?><?php

/**
 * Shipping Module "United Parcel Service"
 *
 * @package ShippingModuleUPS
 * @author Egor V. Derevyankin
 */

class Shipping_Module_UPS extends pm_sm_api
{
//------------------------------------------------
//               PUBLIC DECLARATION
//------------------------------------------------

    /**#@+
     * @access public
     */

    /**
     * ShippingModuleUPS  constructor
     */
    function Shipping_Module_UPS()
    {
        $this->_local_weight_coeff = modApiFunc("Localization","getValue","WEIGHT_COEFF");
    }

	function getUID()
	{
		return "B326C2B6-5709-42C2-9BCC-790A77CB7A0A";
	}

    /**
     * @return Info about this shipping module
     */
    function getInfo()
    {

        global $application;
        $obj = &$application->getInstance('MessageResources',"shipping-module-ups-messages", "AdminZone");

        $this->getSettings();

        return array("GlobalUniqueShippingModuleID" => $this->getUID(),
                     "Name"        => $this->Settings["MODULE_NAME"],
                     "StatusMessage" => ($this->IsActive())? "<span class=\"status_online\">".$obj->getMessage('MODULE_STATUS_ACTIVE')."</span>":$obj->getMessage('MODULE_STATUS_INACTIVE')."<span class=\"required\">*</span>", //NOT for active/inactive.
                     "PreferencesAZViewClassName" => "ups_input_az",
                     "APIClassName" => __CLASS__
                    );
    }

    /**
     * Check requirments
     *
     * @return array with the failed requirments
     */
    function checkRequirments()
    {
        $return=array();

        // check SSL
        loadCoreFile('bouncer.php');
        $bssl = new Bouncer();
        if(!$bssl->isSSLavailable())
            $return[]="SSL";

/*        //check origin country
        $ocntr = modApiFunc("Location","getCountryCode",modApiFunc("Configuration","getValue",SYSCONFIG_STORE_OWNER_COUNTRY));
        if($ocntr!="US")
            $return[]="OCNTR";
*/

/*        //check local currency
        $main_currency_id = modApiFunc("Localization", "getMainStoreCurrency");
        $local_currency_code = modApiFunc("Localization", "getCurrencyCodeById", $main_currency_id);
        if($local_currency_code!="USD")
            $return[]="CURRENCY";
*/

        return $return;
    }

    /**
     * Module installator
     */
    function install()
    {
        _use(dirname(__FILE__).'/includes/install.inc');

        //                                RemovePersonInfoType
        //                   ,          -     CheckoutEditor
        //             PersonInfoType,         , BillingInfo, ShippingInfo,
        //  CustomerInfo, CreditCardInfo, BankAccountInfo, PaymentMethodInfo (
        //             ), ShippingMethodInfo (              ).
        //       ShippingModule'                                                 ,
        //                                     .
        //                    ,                 PersonInfoType          .
        //
        //  PersonInfoType.

        modApiFunc('EventsManager',
               'addEventHandler',
               'RemovePersonInfoTypeEvent',
               'Shipping_Module_UPS',
               'OnRemovePersonInfoType');

        modApiFunc('EventsManager',
                'addEventHandler',
                'OrderCreated',
                'ParcelSplitMessageHandler',
                'onOrderCreated');
    }

    /**
     *                    RemovePersonInfoType.
     *                  ,          -     CheckoutEditor
     *            PersonInfoType,         , BillingInfo, ShippingInfo,
     * CustomerInfo, CreditCardInfo, BankAccountInfo, PaymentMethodInfo (
     *            ), ShippingMethodInfo (              ).
     *      ShippingModule'                                                 ,
     *                                    .
     *                   ,                 PersonInfoType          .
     *
     * PersonInfoType.
     *
     *        UPS                         PersonInfoType:
     * shippingInfo
     *
     * @param string $person_info_type_tag - tag      person info type' ,
     *                      (turn off)   CheckoutFormEditor' .
     *
     * @return mixed - NULL,                       - "          ",       -
     *                                     ,            ,
     *        (                                   ).
     */
    function OnRemovePersonInfoType($person_info_type_tag)
    {
        global $application;

        if($this->isActive() === false)
        {
            //                        -                                .
            $value =  NULL;
        }
        else
        {
            $this->MessageResources = &$application->getInstance('MessageResources',"shipping-module-ups-messages", "AdminZone");
            $msg = $this->MessageResources->getMessage('MODULE_SHIPPING_UPS_API_MSG_REMOVE_PERSON_INFO_TYPE');

            $value = NULL;
            switch($person_info_type_tag)
            {
                case "shippingInfo":
                {
                    $value = $msg;
                    break;
                }
                default:
                {
                    break;
                }
            }
        }
        return $value;
    }


    /**
     * @return true if module is active or false if is not
     */
    function isActive()
    {
        global $application;
        $active_modules = modApiFunc("Checkout","getActiveModules", "shipping");
        return array_key_exists($this->getUid(), $active_modules);
    }

    /**
     * Uninstall the module.
     */
    function uninstall()
    {
        $query = new DB_Table_Delete(Shipping_Module_UPS::getTables());
        global $application;
        $application->db->getDB_Result($query);
    }

    /**
     * @param $method_id ID of shipping method of this module
     * @return true if ID is valid or false if is not
     */
    function isValidShippingMethodId($method_id)
    {
        $check_result = execQuery('SELECT_SM_UPS_METHODS',array('ApiClassName' => __CLASS__, 'method_id' => $method_id));

        if(isset($check_result[0]['id']))
            return true;
        else
            return false;
    }

    /**
     * @param $method_id ID of shipping method of this module
     * @param $and_calc true if need calculate the shipping cost or false if does not need
     * @return array with the shipping method info (id,method_name,method_code,destination,available,cost)
     */
    function getShippingMethodInfo($method_id,$and_calc=false)
    {
        $currency_id = modApiFunc("Localization", "whichCurrencySendOrderToPaymentShippingGatewayIn", ORDER_NOT_CREATED_YET, $this->getUid());

        $select_result = execQuery('SELECT_SM_UPS_METHODS_EXT',array('ApiClassName' => __CLASS__, 'method_id' => $method_id));

        $method=$select_result[0];
        $method["cost"]=PRICE_N_A;

        if($and_calc!=false)
        {
            //$ShippingInfo = modApiFunc("Checkout", "getPrerequisiteValidationResults", "shippingInfo");
            //$pak_weight=modApiFunc("Checkout", "getOrderWeight", "netto");
            $ShippingInfo = modApiFunc("Shipping_Cost_Calculator","getShippingInfo");
            $pak_weight = modApiFunc("Shipping_Cost_Calculator","getPackWeight");
            $rate=$this->getRateFromCache($currency_id, $ShippingInfo,$pak_weight,$method_id);
            if($rate!=false)
                $method["cost"]=$rate;
        };

        return $method;
    }

    /**
     * @return structure of the tables of this module
     */
    function getTables()
    {
        static $tables;

        if (is_array($tables))
        {
            return $tables;
        }

        $tables = array ();

        $settings = 'sm_ups_settings';
        $tables[$settings] = array();
        $tables[$settings]['columns'] = array
            (
                'id'                => $settings.'.sm_ups_setting_id'
               ,'key'               => $settings.'.sm_ups_setting_key'
               ,'value'             => $settings.'.sm_ups_setting_value'
            );
        $tables[$settings]['types'] = array
            (
                'id'                => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'key'               => DBQUERY_FIELD_TYPE_CHAR50
               ,'value'             => DBQUERY_FIELD_TYPE_CHAR255
            );
        $tables[$settings]['primary'] = array
            (
                'id'
            );

        $methods = 'sm_ups_methods';
        $tables[$methods] = array();
        $tables[$methods]['columns'] = array
            (
                'id'                => $methods.'.sm_ups_method_id'
               ,'method_code'       => $methods.'.sm_ups_method_code'
               ,'method_name'       => $methods.'.sm_ups_method_name'
               ,'destination'       => $methods.'.sm_ups_method_destination'
               ,'available'         => $methods.'.sm_ups_method_available'
            );
        $tables[$methods]['types'] = array
            (
                'id'                => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'method_code'       => DBQUERY_FIELD_TYPE_CHAR20
               ,'method_name'       => DBQUERY_FIELD_TYPE_CHAR255
               ,'destination'       => DBQUERY_FIELD_TYPE_CHAR1 .' default \'L\''
               ,'available'         => DBQUERY_FIELD_TYPE_CHAR1 .' default \'N\''
            );
        $tables[$methods]['primary'] = array
            (
                'id'
            );

        $rates_cache = 'sm_ups_rates_cache';
        $tables[$rates_cache] = array();
        $tables[$rates_cache]['columns'] = array
            (
                'id'                => $rates_cache.'.sm_ups_crate_id'
               ,'hash'              => $rates_cache.'.sm_ups_crate_hash'
               ,'method_id'         => $rates_cache.'.sm_ups_crate_method_id'
               ,'rate'              => $rates_cache.'.sm_ups_crate_rate'
               ,'expire'            => $rates_cache.'.sm_ups_crate_expire'
            );
        $tables[$rates_cache]['types'] = array
            (
                'id'                => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'hash'              => DBQUERY_FIELD_TYPE_CHAR50
               ,'method_id'         => DBQUERY_FIELD_TYPE_INT
               ,'rate'              => DBQUERY_FIELD_TYPE_DECIMAL20_5
               ,'expire'            => DBQUERY_FIELD_TYPE_INT
            );
        $tables[$rates_cache]['primary'] = array
            (
                'id'
            );


        global $application;
        return $application->addTablePrefix($tables);
    }

    /**
     *
     */
    function clearSettingsInDB()
    {
        global $application;
        $query = new DB_Delete('sm_ups_settings');
        $application->db->getDB_Result($query);
    }

    /**
     * @return array of the settings of this module
     */
    function getSettings()
    {
        $result = execQuery('SELECT_PM_SM_SETTINGS',array('ApiClassName' => __CLASS__, 'settings_table_name' => 'sm_ups_settings'));
        $this->Settings = array();
        for ($i=0; $i<sizeof($result); $i++)
        {
            $this->Settings[$result[$i]['set_key']] = unserialize($result[$i]['set_value']);
        }
        return $this->Settings;
    }

    /**
     * Set settings for this module
     * @param $Settings array of the all settings of this module
     */
    function setSettings($Settings)
    {
        global $application;
        $this->clearSettingsInDB();
        $tables = $this->getTables();
        $columns = $tables['sm_ups_settings']['columns'];

        foreach($Settings as $key => $value)
        {
            $query = new DB_Insert('sm_ups_settings');
            $query->addInsertValue($key, $columns['key']);
            $query->addInsertValue(serialize($value), $columns['value']);
            $application->db->getDB_Result($query);

            $inserted_id = $application->db->DB_Insert_Id();
        }
    }

    /**
     * Update settings for this module
     * @param $Settings array of the updated settings of this module
     */
    function updateSettings($Settings)
    {
        global $application;
        $tables = $this->getTables();
        $columns = $tables['sm_ups_settings']['columns'];

        foreach($Settings as $key => $value)
        {
            $query = new DB_Update('sm_ups_settings');
            $query->addUpdateValue($columns['value'], serialize($value));
            $query->WhereValue($columns['key'], DB_EQ, $key);
            $application->db->getDB_Result($query);
        }
    }

    /**
     * @param $type type of the requested methods (ALL|AVAILABLE|NOT_AVAILABLE)
     * @param $and_calc true if need calculate the shipping cost or false if does not need
     * @return array of the shipping methods arrays, every will be as (id,method_name,method_code,destination,available,cost)
     */
    function getShippingMethods($type="ALL",$and_calc=false)
    {
        global $application;

        $available_condition=($type=="AVAILABLE"?"'Y'":($type=="NOT_AVAILABLE"?"'N'":""));

        $tables=$this->getTables();

        $methods_table=$tables['sm_ups_methods']['columns'];

        $methods = execQuery('SELECT_SM_UPS_METHODS_BY_AVAILABILITY',array('ApiClassName' => __CLASS__, 'available_condition' => $available_condition));

        if($and_calc!=false and !empty($methods))
        {
            /*
            if($this->is_debug==true)
            {
                $ShippingInfo = modApiFunc("Shipping_Tester","getShippingInfo");
                $pak_weight=modApiFunc("Shipping_Tester","getPackWeight");
            }
            else
            {
                $ShippingInfo = modApiFunc("Checkout", "getPrerequisiteValidationResults", "shippingInfo");
                $pak_weight=modApiFunc("Checkout", "getOrderWeight", "netto");
            }
            */
            $ShippingInfo = modApiFunc("Shipping_Cost_Calculator","getShippingInfo");
            $pak_weight=modApiFunc("Shipping_Cost_Calculator","getPackWeight");

            $_rates=array();
            $cmethods=array();

            if($ShippingInfo["isMet"]!=false)
            {
                $_rates=$this->calculate_rates($ShippingInfo,$pak_weight);
            };

            if(!empty($_rates))
            {
                foreach($_rates as $service => $sdata)
                {
                    foreach($methods as $key => $method)
                    {
                        if($method['method_code']==$service)
                        {
                            $cmethods[]=array_merge($method,$sdata);
                        };
                    };
                };
            };

            $methods=$cmethods;
            $currency_id = modApiFunc("Localization", "whichCurrencySendOrderToPaymentShippingGatewayIn", ORDER_NOT_CREATED_YET, $this->getUid());
            $this->saveRatesToCache($currency_id, $ShippingInfo,$pak_weight,$methods);
        };


        return $methods;
    }

    /**
     * @param $update_array array if the IDs of the available methods,
     *      all other methods will be set to 'not available'
     */
    function updateShippingMethods($update_array)
    {
        global $application;

        $tables=$this->getTables();
        $methods_table=$tables['sm_ups_methods']['columns'];

        $query = new DB_Update('sm_ups_methods');
        $query->addUpdateValue($methods_table['available'],"N");
        $application->db->getDB_Result($query);

        if(count($update_array)==0 or !is_array($update_array))
            return;

        $query = new DB_Update('sm_ups_methods');
        $query->addUpdateValue($methods_table['available'],"Y");
        $query->Where($methods_table['id'], DB_IN, "('".implode("','",array_keys($update_array))."')");
        $application->db->getDB_Result($query);

        return;
    }

    /**
     * @param $s_info shipping info of the destination
     * @param $weight weight of the package
     * @param $rates array of the shipping rates for saving
     */
    function saveRatesToCache($currency_id, $s_info,$weight,$rates)
    {
        if($s_info["isMet"]==false or empty($rates))
            return;

        $s_country=modApiFunc("Location","getCountry",$s_info["validatedData"]["Country"]["value"]);
        $s_zipcode=$s_info["validatedData"]["Postcode"]["value"];
        $s_name = isset($s_info["validatedData"]["Firstname"]["value"]) ? $s_info["validatedData"]["Firstname"]["value"] : '';
        $s_name.= isset($s_info["validatedData"]["Lastname"]["value"])  ? $s_info["validatedData"]["Lastname"]["value"]  : '';

        $hash=md5($currency_id.$s_country.$s_name.$s_zipcode.$weight);

        $tables=$this->getTables();
        $rc_table=$tables['sm_ups_rates_cache']['columns'];

        global $application;

        $query = new DB_Delete('sm_ups_rates_cache');
        $query->Where($rc_table['hash'], DB_EQ, "'$hash'");
        $application->db->getDB_Result($query);

        $query = new DB_Delete('sm_ups_rates_cache');
        $query->Where($rc_table['expire'], DB_LT, time());
        $application->db->getDB_Result($query);

        foreach($rates as $key => $rate_info)
        {
            $query = new DB_Insert('sm_ups_rates_cache');
            $query->addInsertValue($hash, $rc_table["hash"]);
            $query->addInsertValue($rate_info["id"],$rc_table["method_id"]);
            $query->addInsertValue($rate_info["cost"],$rc_table["rate"]);
            $query->addInsertValue(time()+18000,$rc_table["expire"]);
            $application->db->getDB_Result($query);
        };

        return;
    }

    /**
     * @param $s_info shipping info of the destination
     * @param $weight weight of the package
     * @param $method_id ID of the needed shipping method
     * @return cost of the shipping from the cache
     *      or false if the requested cost is not saved in the cache
     */
    /*
     *      !
     *                                        ,
     *                             .
     *                                            UPS
     *         UPS,                             ,
     *                      .                          -           .
     *                              ,
     *         UPS        .
     */
    function getRateFromCache($currency_id, $s_info,$weight,$method_id)
    {
        if($s_info["isMet"]==false or !is_numeric($method_id))
            return false;

        $s_country=modApiFunc("Location","getCountry",$s_info["validatedData"]["Country"]["value"]);
        $s_zipcode=$s_info["validatedData"]["Postcode"]["value"];
        $s_name = isset($s_info["validatedData"]["Firstname"]["value"]) ? $s_info["validatedData"]["Firstname"]["value"] : '';
        $s_name.= isset($s_info["validatedData"]["Lastname"]["value"])  ? $s_info["validatedData"]["Lastname"]["value"]  : '';

        $hash=md5($currency_id.$s_country.$s_name.$s_zipcode.$weight);

        $cached = execQuery('SELECT_SM_UPS_METHODS_CACHED',array('ApiClassName' => __CLASS__, 'hash' => $hash, 'method_id' => $method_id));

        if(isset($cached[0]["rate"]))
            return $cached[0]["rate"];
        else
            return false;
    }

    /**
     * @param $ShippingInfo shipping info of the destination
     * @param $pak_weight weight of the package
     * @return array of the shipping rates arrays
     */
    function calculate_rates($ShippingInfo, $pak_weight)
    {
        global $application;
        if($this->is_debug==true)
            $this->_debug_info[]=array("Calculate Type","Real-Time");

        $_rates = array();

        # get Settings
        $Settings = $this->getSettings();

        # define the 'Customer classification code' by the 'Pickup type'
        if($Settings["PICKUP_TYPE"]=="01")
            $customer_classification_code="01";
        elseif(in_array($Settings["PICKUP_TYPE"],array("06","07","19","20")))
            $customer_classification_code="03";
        elseif(in_array($Settings["PICKUP_TYPE"],array("03","11")))
            $customer_classification_code="04";

        # define the origination and destination city, zipcode and country
        $org_city = modApiFunc("Configuration","getTagValue","StoreOwnerCity");
        $org_zip = modApiFunc("Configuration","getTagValue","StoreOwnerPostcode");
        $org_cntr = modApiFunc("Location","getCountryCode",modApiFunc("Configuration","getValue",SYSCONFIG_STORE_OWNER_COUNTRY));
        $dst_city = isset($ShippingInfo["validatedData"]["City"]["value"]) ? $ShippingInfo["validatedData"]["City"]["value"] : '';
        $dst_zip = $ShippingInfo["validatedData"]["Postcode"]["value"];
        $dst_cntr = modApiFunc("Location","getCountryCode",$ShippingInfo["validatedData"]["Country"]["value"]);

        # Calculate weight
        $weight_in_lbs = $this->_convertWeightToPounds($pak_weight);

        if ($weight_in_lbs < 1)
            $weight_in_lbs = 1;

        # Get the declared value of package
        $decl_value = modApiFunc("Shipping_Cost_Calculator","getCartSubtotal");
        //Shipping_Cost_Calculator                 Main Store Currency,  . .
        //
        //                                                    ,
        //                       .
        //
        //         (                   ),              (   )    . .
        $currency_id = modApiFunc("Localization", "whichCurrencySendOrderToPaymentShippingGatewayIn", ORDER_NOT_CREATED_YET, $this->getUid());
        $main_store_currency_id = modApiFunc("Localization", "getMainStoreCurrency");

        $main_store_currency_code = modApiFunc("Localization", "getCurrencyCodeById", $main_store_currency_id);
        $currency_code = modApiFunc("Localization", "getCurrencyCodeById", $currency_id);
        if ($currency_id != $main_store_currency_id)
        {
            $decl_value = modApiFunc("Currency_Converter", "convert", $decl_value, $main_store_currency_code, $currency_code);
        }

        if ($this->is_debug==true)
        {
            $this->_debug_info[]=array("Destination Country Code",$dst_cntr);
            $this->_debug_info[]=array("Destination Zip",$dst_zip);
            $this->_debug_info[]=array("Residental delivery",$Settings["SHIP2RES"]);
            $this->_debug_info[]=array("Service URL is","https://www.ups.com/ups.app/xml/Rate");
        }

    //========================= dimensions prerequest
        if ($Settings["PK_TYPE"] == "02")
        {
            $fields = array(
                         "PK_LENGTH" => $Settings["PK_LENGTH"]
                        ,"PK_WIDTH"  => $Settings["PK_WIDTH"]
                        ,"PK_HEIGHT" => $Settings["PK_HEIGHT"]
                );
            $xml_template = file_get_contents(dirname(__FILE__)."/includes/dimensions_request.xml");
            $dim_request = strtr($xml_template, $fields);
        }
        else
        {
            $dim_request = "";
        }
    //=================================================

    //============================== package request
        $fields = array(
                 "PK_TYPE"              => $Settings["PK_TYPE"]
                ,"CURRENCY_CODE"        => $currency_code
                ,"<Dimensions/>"        => $dim_request
            );
        $xml_template = file_get_contents(dirname(__FILE__)."/includes/package_request.xml");
        $pack_request = strtr($xml_template, $fields);
    //===============================================

        // weight check
        $packages = "";
        if ($weight_in_lbs <= $Settings["WEIGHT_THRESHOLD"])
        {
            // particular request
            $fields = array(
                         "WEIGHT_IN_LBS" => $weight_in_lbs
                        ,"DECL_VALUE"    => $decl_value
                );
            $packages .= strtr($pack_request, $fields);
        }
        else
        {
            $threshold = $this->_convertWeightFromPounds($Settings["WEIGHT_THRESHOLD"]);
            $order = modApiFunc("Cart", "getCartContent");
            $parcels = modApiFunc("ShippingCompositor", "split_cart", $order, $threshold);

            $msgRes = &$application->getInstance('MessageResources',"shipping-module-ups-messages", "AdminZone");
            $msg_splitted = $msgRes->getMessage('PARCEL_SPLITTED_MSG') . "<br/>";
            $num = count($parcels);
            if (isset($parcels["total_weight"])) $num--;
            if (isset($parcels["subtotal"])) $num--;
            if (isset($parcels["msg"])) $num--;
            $msg_splitted = str_replace("AMOUNT", $num, $msg_splitted);

            foreach ($parcels as $key => $value)
            {
                if ($key === "total_weight"
                    || $key === "subtotal"
                    || $key === "msg")
                    continue;

                // particular request
                $fields = array(
                             "WEIGHT_IN_LBS" => $this->_convertWeightToPounds($value["total_weight"])
                            ,"DECL_VALUE"    => modApiFunc("Currency_Converter", "convert", $value["subtotal"], $main_store_currency_code, $currency_code)
                    );
                $packages .= strtr($pack_request, $fields);

                $msg_splitted .= "Parcel #" . ($key + 1) . "<br/>";
                foreach ($value["products"] as $k => $v)
                {
                    $msg_splitted .= "&nbsp;&nbsp;&nbsp;&nbsp;".($k+1)."&nbsp;{$v['Name']}&nbsp;{$v['SKU']}&nbsp;{$v['qty']}pcs.*{$v['cost']}&nbsp;W:&nbsp;{$v['weight']}&nbsp;lb&nbsp;GW:&nbsp;" . ($v['qty']*$v['weight']) . "&nbsp;lb";
                    if (isset($v["warning"]) && $v["warning"] == true)
                        $msg_splitted .= "&nbsp;overweight";
                    $msg_splitted .= "<br/>";
                }
                $msg_splitted .= "&nbsp;&nbsp;Parcel GW: " . $parcels[$key]["total_weight"] . "&nbsp;lb<br/><br/>";
            }

            $msg_splitted .= "Total weight: " . $parcels["total_weight"];
            modApiFunc("Session", 'set', 'ShippingModuleParcelSplit_' . $this->getUID(), $msg_splitted);
        }

    //========================= general request
        $fields = array(
                     "ACCESSKEY"            => $Settings["ACCESSKEY"]
                    ,"USERNAME"             => $Settings["USERNAME"]
                    ,"PASSWORD"             => $Settings["PASSWORD"]
                    ,"PICKUP_TYPE"          => $Settings["PICKUP_TYPE"]
                    ,"CUSTOMER_CLASSIFICATION_CODE" => $customer_classification_code
                    ,"ORIG_CITY"            => $org_city
                    ,"ORIG_ZIP"             => $org_zip
                    ,"ORIG_COUNTRY_CODE"    => $org_cntr
                    ,"DEST_CITY"            => $dst_city
                    ,"DEST_ZIP"             => $dst_zip
                    ,"DEST_COUNTRY"         => $dst_cntr
                    ,"<ResidentialAddressIndicator/>" => ($Settings["SHIP2RES"]=="YES") ? "<ResidentialAddressIndicator/>" : ""
                    ,"<Package />"           => $packages
             );
        $xml_template = file_get_contents(dirname(__FILE__)."/includes/general_request.xml");
        $gen_request = strtr($xml_template, $fields);
    //============================================

        $xml_request = $gen_request;

        if ($this->is_debug==true)
            $this->_debug_info[]=array("XML Request",$xml_request);

        # create secure connection to the UPS server and send XML-request
        loadCoreFile('bouncer.php');
        $bnc = new Bouncer();
        $bnc->setMethod("POST");
        $bnc->setURL("https://www.ups.com/ups.app/xml/Rate");
        $bnc->setPOSTstring($xml_request);

        $res = $bnc->RunRequest();

        if($res!=false) # if we have the response
        {
            # get the XML-response
            $xml_response=$res["body"];

            if($this->is_debug==true)
                $this->_debug_info[]=array("XML Response",$xml_response);

            # parse the XML-response
            loadCoreFile('obj_xml.php');
            $parser = new xml_doc($xml_response);
            $parser->parse();

            foreach($parser->xml_index[0]->children as $k => $iChild)
            {
                if($iChild->name == "RATEDSHIPMENT")
                {
                    $_service = ""; # service code
                    $_rate = "";    # service rate
                    $_rate_currency_code = $currency_code;
                    foreach($iChild->children as $k_ic => $rsChild)
                        if($rsChild->name=="SERVICE")
                        {
                            foreach($rsChild->children as $k_rs => $sChild)
                                if($sChild->name=="CODE")        # get code
                                    $_service=$sChild->contents;
                        }
                        elseif($rsChild->name=="TOTALCHARGES")
                        {
                            foreach($rsChild->children as $k_rs => $tcChild)
                                if($tcChild->name=="MONETARYVALUE") # get rate
                                    $_rate=$tcChild->contents;
                                elseif($tcChild->name=="CURRENCYCODE")
			                        {
			                            $_rate_currency_code = $tcChild->contents;
			                        }

                        };
                    # put received values into rates array

                    //           ,
                    //    Main Store Currency,
			        if($_rate_currency_code != $main_store_currency_code)
			        {
			            $_rate = modApiFunc("Currency_Converter", "convert", $_rate, $_rate_currency_code, $main_store_currency_code);
			        }

                    $_rates[$_service]=array("cost"=>$_rate,"days"=>0);
                }
                elseif($iChild->name=="ERROR" and $this->is_debug==true)
                {
                    $_emsg="";
                    foreach($iChild->children as $erk => $eChild)
                        if(in_array($eChild->name,array("ERRORSEVERITY","ERRORCODE","ERRORDESCRIPTION")))
                            $_emsg.=$eChild->contents.":";
                        elseif($eChild->name=="ERRORLOCATION")
                            foreach($eChild->children as $elk => $elChild)
                                if($elChild->name=="ERRORLOCATIONELEMENTNAME")
                                    $_emsg.=$elChild->contents;
                    $this->_debug_info[]=array("Error",$_emsg);
                }
            }
        }
        elseif($this->is_debug==true)
            $this->_debug_info[]=array("Error",$bnc->getLastErrorMessage());

        return $_rates;
    }

    function setDebugFlag($flag=true)
    {
        if(is_bool($flag))
            $this->is_debug=$flag;
    }

    function getDebugInfo()
    {
        return $this->_debug_info;
    }

    function _convertWeightToPounds($weight)
    {
        return round(($weight * $this->_local_weight_coeff * 1000 / 453.6), 1);
    }

    function _convertWeightFromPounds($weight)
    {
        return round(($weight * (453.6 / 1000) / $this->_local_weight_coeff), 1);
    }

    /**#@-*/

//------------------------------------------------
//              PRIVATE DECLARATION
//------------------------------------------------

    /**#@+
     * @access private
     */
    var $bActive;

    var $is_debug;
    var $_debug_info;
    var $_local_weight_coeff;

    /**#@-*/
}
?>