<?php
/***********************************************************************
| Avactis (TM) Shopping Cart software developed by Pentasoft Corp.
| http://www.avactis.com
| -----------------------------------------------------------------------
| All source codes & content (c) Copyright 2004-2010, Pentasoft Corp.
| unless specifically noted otherwise.
| =============================================
| This source code is released under the Avactis License Agreement.
| The latest version of this license can be found here:
| http://www.avactis.com/license.php
|
| By using this software, you acknowledge having read this license agreement
| and agree to be bound thereby.
|
 ***********************************************************************/
?><?php

/**
 * Shipping Module "U.S. Postal Service".
 *
 * @package ShippingModuleUSPS
 * @author Egor V. Derevyankin
 */
class Shipping_Module_USPS extends pm_sm_api
{
//------------------------------------------------
//               PUBLIC DECLARATION
//------------------------------------------------

    /**#@+
     * @access public
     */

    /**
     * ShippingModuleUSPS constructor.
     */
    function Shipping_Module_USPS()
    {
    }

	function getUID()
	{
		return "573331CD-E3FE-4FDA-BBF7-B90E2198938D";
	}

    /**
     * @return Info about this shipping module
     */
    function getInfo()
    {

        global $application;
        $obj = &$application->getInstance('MessageResources',"shipping-module-usps-messages", "AdminZone");

        $this->getSettings();

        return array("GlobalUniqueShippingModuleID" => $this->getUID(),
                     "Name"        => $this->Settings["MODULE_NAME"],
                     "StatusMessage" => ($this->IsActive())? "<span class=\"status_online\">".$obj->getMessage('MODULE_STATUS_ACTIVE')."</span>":$obj->getMessage('MODULE_STATUS_INACTIVE')."<span class=\"required\">*</span>", //NOT for active/inactive.
                     "PreferencesAZViewClassName" => "usps_input_az",
                     "APIClassName" => __CLASS__
                    );
    }

    /**
     * Check requirments
     *
     * @return array with the failed requirments
     */
    function checkRequirments()
    {
        $return=array();

        //check origin country
        $ocntr = modApiFunc("Location","getCountryCode",modApiFunc("Configuration","getValue",SYSCONFIG_STORE_OWNER_COUNTRY));
        if($ocntr!="US")
            $return[]="OCNTR";

        //check local currency
        $main_currency_id = modApiFunc("Localization", "getMainStoreCurrency");
        $local_currency_code = modApiFunc("Localization", "getCurrencyCodeById", $main_currency_id);
        if($local_currency_code!="USD")
            $return[]="CURRENCY";

        return $return;
    }


    /**
     * Module installator.
     */
    function install()
    {
        _use(dirname(__FILE__).'/includes/install.inc');

        //                                RemovePersonInfoType
        //                   ,          -     CheckoutEditor
        //             PersonInfoType,         , BillingInfo, ShippingInfo,
        //  CustomerInfo, CreditCardInfo, BankAccountInfo, PaymentMethodInfo (
        //             ), ShippingMethodInfo (              ).
        //       ShippingModule'                                                 ,
        //                                     .
        //                    ,                 PersonInfoType          .
        //
        //  PersonInfoType.
        modApiFunc('EventsManager',
               'addEventHandler',
               'RemovePersonInfoTypeEvent',
               'Shipping_Module_USPS',
               'OnRemovePersonInfoType');
    }

    /**
     *                    RemovePersonInfoType.
     *                  ,          -     CheckoutEditor
     *            PersonInfoType,         , BillingInfo, ShippingInfo,
     * CustomerInfo, CreditCardInfo, BankAccountInfo, PaymentMethodInfo (
     *            ), ShippingMethodInfo (              ).
     *      ShippingModule'                                                 ,
     *                                    .
     *                   ,                 PersonInfoType          .
     *
     * PersonInfoType.
     *
     *        USPS                         PersonInfoType:
     * shippingInfo
     *
     * @param string $person_info_type_tag - tag      person info type' ,
     *                      (turn off)   CheckoutFormEditor' .
     *
     * @return mixed - NULL,                       - "          ",       -
     *                                     ,            ,
     *        (                                   ).
     */
    function OnRemovePersonInfoType($person_info_type_tag)
    {
        global $application;

        if($this->isActive() === false)
        {
            //                        -                                .
            $value =  NULL;
        }
        else
        {
            $this->MessageResources = &$application->getInstance('MessageResources',"shipping-module-usps-messages", "AdminZone");
            $msg = $this->MessageResources->getMessage('MODULE_SHIPPING_USPS_API_MSG_REMOVE_PERSON_INFO_TYPE');

            $value = NULL;
            switch($person_info_type_tag)
            {
                case "shippingInfo":
                {
                    $value = $msg;
                    break;
                }
                default:
                {
                    break;
                }
            }
        }
        return $value;
    }


    /**
     * @return true if module is active or false if it is not
     */
    function isActive()
    {
        global $application;
        $active_modules = modApiFunc("Checkout","getActiveModules", "shipping");
        return array_key_exists($this->getUid(), $active_modules);
    }

    /**
     * Uninstalls the module.
     */
    function uninstall()
    {
        $query = new DB_Table_Delete(Shipping_Module_USPS::getTables());
        global $application;
        $application->db->getDB_Result($query);
    }

    /**
     * @param $method_id ID of shipping method of this module
     * @return true if ID is valid or false if it is not
     */
    function isValidShippingMethodId($method_id)
    {
        global $application;
        $tables=$this->getTables();
        $methods_table=$tables['sm_usps_methods']['columns'];

        $check_result = execQuery('SELECT_SM_USPS_METHODS',array('ApiClassName' => __CLASS__, 'method_id' => $method_id));
        if(isset($check_result[0]['id']))
            return true;
        else
            return false;
    }

    /**
     * @param $method_id ID of shipping method of this module
     * @param $and_calc true if it is necessary to calculate the shipping cost
     *        or false if it is not
     * @return array with the shipping method info (id,method_name,method_code,
     * destination,available,cost)
     */
    function getShippingMethodInfo($method_id,$and_calc=false)
    {
        global $application;
        $tables=$this->getTables();
        $methods_table=$tables['sm_usps_methods']['columns'];

        $select_result = execQuery('SELECT_SM_USPS_METHODS_EXT',array('ApiClassName' => __CLASS__, 'method_id' => $method_id));

        $method=$select_result[0];
        $method["cost"]=PRICE_N_A;

        if($and_calc!=false)
        {
            //$ShippingInfo = modApiFunc("Checkout", "getPrerequisiteValidationResults", "shippingInfo");
            //$pak_weight=modApiFunc("Checkout", "getOrderWeight", "netto");
            $ShippingInfo = modApiFunc("Shipping_Cost_Calculator","getShippingInfo");
            $pak_weight = modApiFunc("Shipping_Cost_Calculator","getPackWeight");
            $rate=$this->getRateFromCache($ShippingInfo,$pak_weight,$method_id);
            if($rate!=false)
                $method["cost"]=$rate;
        };

        return $method;
    }

    /**
     * @return structure of the tables of this module
     */
    function getTables()
    {

        static $tables;

        if (is_array($tables))
        {
            return $tables;
        }

        $tables = array ();

        $settings = 'sm_usps_settings';
        $tables[$settings] = array();
        $tables[$settings]['columns'] = array
            (
                'id'                => $settings.'.sm_usps_setting_id'
               ,'key'               => $settings.'.sm_usps_setting_key'
               ,'value'             => $settings.'.sm_usps_setting_value'
            );
        $tables[$settings]['types'] = array
            (
                'id'                => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'key'               => DBQUERY_FIELD_TYPE_CHAR50
               ,'value'             => DBQUERY_FIELD_TYPE_CHAR255
            );
        $tables[$settings]['primary'] = array
            (
                'id'
            );

        $methods = 'sm_usps_methods';
        $tables[$methods] = array();
        $tables[$methods]['columns'] = array
            (
                'id'                => $methods.'.sm_usps_method_id'
               ,'method_code'       => $methods.'.sm_usps_method_code'
               ,'method_name'       => $methods.'.sm_usps_method_name'
               ,'destination'       => $methods.'.sm_usps_method_destination'
               ,'available'         => $methods.'.sm_usps_method_available'
            );
        $tables[$methods]['types'] = array
            (
                'id'                => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'method_code'       => DBQUERY_FIELD_TYPE_CHAR5
               ,'method_name'       => DBQUERY_FIELD_TYPE_CHAR255
               ,'destination'       => DBQUERY_FIELD_TYPE_CHAR1 .' default \'L\''
               ,'available'         => DBQUERY_FIELD_TYPE_CHAR1 .' default \'N\''
            );
        $tables[$methods]['primary'] = array
            (
                'id'
            );

        $rates_cache = 'sm_usps_rates_cache';
        $tables[$rates_cache] = array();
        $tables[$rates_cache]['columns'] = array
            (
                'id'                => $rates_cache.'.sm_usps_crate_id'
               ,'hash'              => $rates_cache.'.sm_usps_crate_hash'
               ,'method_id'         => $rates_cache.'.sm_usps_crate_method_id'
               ,'rate'              => $rates_cache.'.sm_usps_crate_rate'
               ,'expire'            => $rates_cache.'.sm_usps_crate_expire'
            );
        $tables[$rates_cache]['types'] = array
            (
                'id'                => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'hash'              => DBQUERY_FIELD_TYPE_CHAR50
               ,'method_id'         => DBQUERY_FIELD_TYPE_INT
               ,'rate'              => DBQUERY_FIELD_TYPE_DECIMAL20_5
               ,'expire'            => DBQUERY_FIELD_TYPE_INT
            );
        $tables[$rates_cache]['primary'] = array
            (
                'id'
            );


        global $application;
        return $application->addTablePrefix($tables);
    }

    /**
     *
     */
    function clearSettingsInDB()
    {
        global $application;
        $query = new DB_Delete('sm_usps_settings');
        $application->db->getDB_Result($query);
    }

    /**
     * @return array of the settings of this module
     */
    function getSettings()
    {
        $result = execQuery('SELECT_PM_SM_SETTINGS',array('ApiClassName' => __CLASS__, 'settings_table_name' => 'sm_usps_settings'));
        $this->Settings = array();
        for ($i=0; $i<sizeof($result); $i++)
        {
            $this->Settings[$result[$i]['set_key']] = unserialize($result[$i]['set_value']);
        }
        return $this->Settings;
    }

    /**
     * Sets settings for this module.
     *
     * @param $Settings array of the all settings of this module
     */
    function setSettings($Settings)
    {
        global $application;
        $this->clearSettingsInDB();
        $tables = $this->getTables();
        $columns = $tables['sm_usps_settings']['columns'];

        foreach($Settings as $key => $value)
        {
            $query = new DB_Insert('sm_usps_settings');
            $query->addInsertValue($key, $columns['key']);
            $query->addInsertValue(serialize($value), $columns['value']);
            $application->db->getDB_Result($query);

            $inserted_id = $application->db->DB_Insert_Id();
        }
    }

    /**
     * Updates settings for this module.
     *
     * @param $Settings array of the updated settings of this module
     */
    function updateSettings($Settings)
    {

        global $application;
        $tables = $this->getTables();
        $columns = $tables['sm_usps_settings']['columns'];

        foreach($Settings as $key => $value)
        {
            $query = new DB_Update('sm_usps_settings');
            $query->addUpdateValue($columns['value'], serialize($value));
            $query->WhereValue($columns['key'], DB_EQ, $key);
            $application->db->getDB_Result($query);
        }

    }

    /**
     * @param $type type of the requested methods (ALL|AVAILABLE|NOT_AVAILABLE)
     * @param $and_calc true if it is necessary to calculate the shipping cost
     *        or false if it is not
     * @return array of the shipping methods arrays, every will be as
     *        (id,method_name,method_code,destination,available,cost)
     */
    function getShippingMethods($type="ALL", $and_calc=false)
    {
        global $application;

        $available_condition = '';
        if ($type == "AVAILABLE")
            $available_condition = "'Y'";
        else if ($type == "NOT_AVAILABLE")
            $available_condition = "'N'";

        $tables = $this->getTables();
        $methods_table = $tables['sm_usps_methods']['columns'];

        $methods = execQuery('SELECT_SM_USPS_METHODS_BY_AVAILABILITY',array('ApiClassName' => __CLASS__, 'available_condition' => $available_condition));

        if($and_calc!=false and !empty($methods))
        {
            $ShippingInfo = modApiFunc("Shipping_Cost_Calculator","getShippingInfo");
            $pak_weight = modApiFunc("Shipping_Cost_Calculator","getPackWeight");

            $_rates = array();
            $cmethods = array();

            if($ShippingInfo["isMet"]!=false)
            {
                $ocntr = modApiFunc("Location","getCountryCode",modApiFunc("Configuration","getValue",SYSCONFIG_STORE_OWNER_COUNTRY));
                $dcntr = modApiFunc("Location","getCountryCode",$ShippingInfo["validatedData"]["Country"]["value"]);
                $_rates=$this->calculate_rates($ShippingInfo,$pak_weight);
            };

            if(!empty($_rates))
            {
                $ForNational = ($ocntr == ($dcntr) ? true : false);

                // clean unnecessary domestic / international methods
                foreach ($methods as $key => $value)
                {
                    if ($ForNational && $value['destination'] == 'I')
                        continue;
                    if (!$ForNational && $value['destination'] == 'L')
                        continue;

                    $m_arr[$value["method_code"]] = $value;
                }

                foreach($_rates as $rate_id => $rate_data)
                {
                    if (isset($m_arr[$rate_id])
                        && $m_arr[$rate_id]['available'] == 'Y')
                    {
                        $cmethods[] = array_merge($rate_data, $m_arr[$rate_id]);
                    }
                }
            }

            $methods = $cmethods;
            $this->saveRatesToCache($ShippingInfo, $pak_weight, $cmethods);
        }

        return $methods;
    }

    /**
     * @param $update_array array if the IDs of the available methods,
     *      all other methods will be set to 'not available'
     */
    function updateShippingMethods($update_array)
    {
        global $application;

        $tables=$this->getTables();
        $methods_table=$tables['sm_usps_methods']['columns'];

        $query = new DB_Update('sm_usps_methods');
        $query->addUpdateValue($methods_table['available'],"N");
        $application->db->getDB_Result($query);

        if(count($update_array)==0 or !is_array($update_array))
            return;

        $query = new DB_Update('sm_usps_methods');
        $query->addUpdateValue($methods_table['available'],"Y");
        $query->Where($methods_table['id'], DB_IN, "('".implode("','",array_keys($update_array))."')");
        $application->db->getDB_Result($query);

        return;
    }

    /**
     * @param $s_info shipping info of the destination
     * @param $weight weight of the package
     * @param $rates array of the shipping rates for saving
     */
    function saveRatesToCache($s_info,$weight,$rates)
    {
        if($s_info["isMet"]==false or empty($rates))
            return;

        $s_country=modApiFunc("Location","getCountry",$s_info["validatedData"]["Country"]["value"]);
        $s_zipcode=$s_info["validatedData"]["Postcode"]["value"];
        $s_name = isset($s_info["validatedData"]["Firstname"]["value"]) ? $s_info["validatedData"]["Firstname"]["value"] : '';
        $s_name.= isset($s_info["validatedData"]["Lastname"]["value"])  ? $s_info["validatedData"]["Lastname"]["value"]  : '';

        $hash=md5($s_country.$s_name.$s_zipcode.$weight);

        $tables=$this->getTables();
        $rc_table=$tables['sm_usps_rates_cache']['columns'];

        global $application;

        $query = new DB_Delete('sm_usps_rates_cache');
        $query->Where($rc_table['hash'], DB_EQ, "'$hash'");
        $application->db->getDB_Result($query);

        $query = new DB_Delete('sm_usps_rates_cache');
        $query->Where($rc_table['expire'], DB_LT, time());
        $application->db->getDB_Result($query);

        foreach($rates as $key => $rate_info)
        {
            $query = new DB_Insert('sm_usps_rates_cache');
            $query->addInsertValue($hash, $rc_table["hash"]);
            $query->addInsertValue($rate_info["id"],$rc_table["method_id"]);
            $query->addInsertValue($rate_info["cost"],$rc_table["rate"]);
            $query->addInsertValue(time()+18000,$rc_table["expire"]);
            $application->db->getDB_Result($query);
        };

        return;
    }

    /**
     * @param $s_info shipping info of the destination
     * @param $weight weight of the package
     * @param $method_id ID of the required shipping method
     * @return cost of the shipping from the cache
     *      or false if the requested cost is not saved in the cache
     */
    function getRateFromCache($s_info,$weight,$method_id)
    {
        if($s_info["isMet"]==false or !is_numeric($method_id))
            return false;

        $s_country=modApiFunc("Location","getCountry",$s_info["validatedData"]["Country"]["value"]);
        $s_zipcode=$s_info["validatedData"]["Postcode"]["value"];
        $s_name = isset($s_info["validatedData"]["Firstname"]["value"]) ? $s_info["validatedData"]["Firstname"]["value"] : '';
        $s_name.= isset($s_info["validatedData"]["Lastname"]["value"])  ? $s_info["validatedData"]["Lastname"]["value"]  : '';

        $hash=md5($s_country.$s_name.$s_zipcode.$weight);

        $cached = execQuery('SELECT_SM_USPS_METHODS_CACHED',array('ApiClassName' => __CLASS__, 'hash' => $hash, 'method_id' => $method_id));

        if(isset($cached[0]["rate"]))
            return $cached[0]["rate"];
        else
            return false;
    }

    /**
     * @param $ShippingInfo shipping info of the destination
     * @param $pak_weight weight of the package
     * @return array of the shipping rates arrays
     */
    function calculate_rates($ShippingInfo,$pak_weight)
    {
        if($this->is_debug==true)
            $this->_debug_info[]=array("Calculate Type","Real-Time");

        $_rates=array();

        # get module Settings
        $Settings=$this->getSettings();

        # get codes of the origin country and destination country
        $ocntr = modApiFunc("Location","getCountryCode",modApiFunc("Configuration","getValue",SYSCONFIG_STORE_OWNER_COUNTRY));
        $dcntr = modApiFunc("Location","getCountryCode",$ShippingInfo["validatedData"]["Country"]["value"]);

        # get name of the destination country
        # : create inner table for the country names in english
        $DCountryName=modApiFunc("Location","getCountry",$ShippingInfo["validatedData"]["Country"]["value"]);

        # calculate pakage weight in gramms
        $local_wcoeff = modApiFunc("Localization","getValue","WEIGHT_COEFF");
        $weight_in_gramms = round($pak_weight * $local_wcoeff * 1000);

        # calculate pakage weight in ounces
        $pounds=0;
        $ounces=round($weight_in_gramms/28.35);
        if($ounces < 1)
            $ounces = 1;

        # set settings for the calculation
        $mailtype=$Settings['RT_INT_MAILTYPE'];
        $package_size=$Settings['RT_DOM_PACKSIZE'];
        $machinable=$Settings['RT_DOM_MACHINABLE'];
        $Servername=$Settings['RT_SERVER_NAME'];
        $Username=$Settings['RT_USER_NAME'];

        if($this->is_debug==true)
        {
            $this->_debug_info[]=array("Destination Country Code",$dcntr);
            $this->_debug_info[]=array("Destination Zip",$ShippingInfo["validatedData"]["Postcode"]["value"]);
        }

        # generate XML request
        if($ocntr!=$dcntr) # for international delivery
        {
            $xml_request="".
            "<IntlRateRequest USERID=\"$Username\">".
                "<Package ID=\"0\">".
                    "<Pounds>$pounds</Pounds>".
                    "<Ounces>$ounces</Ounces>".
                    "<MailType>$mailtype</MailType>".
                    "<Country>$DCountryName</Country>".
                "</Package>".
            "</IntlRateRequest>";

            $API_name="IntlRate";
            $parse_service_container="SERVICE";
            $parse_service_name="SVCDESCRIPTION";
            $parse_service_rate="POSTAGE";
            $parse_service_days="SVCCOMMITMENTS";
        }
        else    # for domestic delivery
        {
            $OWNzip=modApiFunc("Configuration","getTagValue","StoreOwnerPostcode");
            $DSTzip=$ShippingInfo["validatedData"]["Postcode"]["value"];

            $xml_request="".
            "<RateV3Request USERID=\"$Username\">".
                "<Package ID=\"0\">".
                    "<Service>All</Service>".
                    "<ZipOrigination>$OWNzip</ZipOrigination>".
                    "<ZipDestination>$DSTzip</ZipDestination>".
                    "<Pounds>$pounds</Pounds>".
                    "<Ounces>$ounces</Ounces>".
                    "<Size>$package_size</Size>".
                    "<Machinable>$machinable</Machinable>".
                "</Package>".
            "</RateV3Request>";

            $API_name="RateV3";
            $parse_service_container="POSTAGE";
            $parse_service_name="MAILSERVICE";
            $parse_service_rate="RATE";
            $parse_service_days="";
        };

        # create GET request for the remote calculation server
        $getRequest=$Servername."/ShippingAPI.dll?API=".$API_name."&XML=".urlencode($xml_request);

        if($this->is_debug==true)
        {
            $this->_debug_info[]=array("Service URL is","http://".$Servername."/ShippingAPI.dll?API=".$API_name);
            $this->_debug_info[]=array("XML Request",$xml_request);
        }

        # send request to remote server and receive response
        loadCoreFile('bouncer.php');
        global $application;
        $bnc = new Bouncer();
        $bnc->setMethod("GET");
        $bnc->setHTTPversion($application->getAppIni('USPS_HTTP_PROTOCOL_VERSION'));
        $bnc->setURL('http://'.$getRequest);
        $result = $bnc->RunRequest();

        $usps_methods = array();
        # if we do not have the right result then return the empty array
        if ($result != false)
        {
            # receive XML responce from the server response
            $xml_response=$result["body"];

            if($this->is_debug==true)
                $this->_debug_info[]=array("XML Response",$xml_response);

            # parse the XML response
            loadCoreFile('obj_xml.php');
            $parser = new xml_doc($xml_response);
            $parser->parse();

            # receive pakage info from the parse result
            $package=$parser->xml_index[0]->children[0]->children;

            # receive services info from the pakage info
            foreach($package as $k => $pinfo)
            {
                if($pinfo->name==$parse_service_container)
                {
                    $_service="";
                    $_rate="";
                    $_days=0;

                    $class_id = 0;
                    if (isset($pinfo->attributes['CLASSID']))
                        $class_id = $pinfo->attributes['CLASSID'];
                    if (isset($pinfo->attributes['ID']))
                        $class_id = $pinfo->attributes['ID'];

                    foreach($pinfo->children as $ck => $cinfo)
                    {
                        if($cinfo->name==$parse_service_name)
                            $_service=$cinfo->contents;
                        else if($cinfo->name==$parse_service_rate)
                            $_rate=$cinfo->contents;
                        else if($cinfo->name==$parse_service_days)
                            $_days=$cinfo->contents;
                    }
                    $_rates[$_service]=array("cost"=>$_rate,"days"=>$_days);

                    if($class_id == "0" && !strpos($_service, $mailtype)) continue;

                    $usps_methods[$class_id] = array(
                        "name" => $_service,
                        "cost" => $_rate,
                        "days" => $_days
                    );
                }
                elseif($pinfo->name=="ERROR" and $this->is_debug==true)
                {
                    $_emsg="";
                    foreach($pinfo->children as $erk => $erChild)
                        $_emsg.=$erChild->contents.":";
                    $this->_debug_info[]=array("Error",_ml_substr($_emsg,0,-1));
                }
            };
        }
        elseif($this->is_debug==true)
            $this->_debug_info[]=array("Error",$bnc->getLastErrorMessage());

        return $usps_methods;
    }

    function setDebugFlag($flag=true)
    {
        if(is_bool($flag))
            $this->is_debug=$flag;
    }

    function getDebugInfo()
    {
        return $this->_debug_info;
    }

    /**#@-*/


//------------------------------------------------
//              PRIVATE DECLARATION
//------------------------------------------------

    /**#@+
     * @access private
     */

    /**
     * @param $method_id ID of shipping method of this module
     * @return shipping cost or PRICE_N_A
     */
    function getShippingCost($method_id="")
    {
        $ShippingInfo = modApiFunc("Checkout", "getPrerequisiteValidationResults", "shippingInfo");
        $pak_weight=modApiFunc("Checkout", "getOrderWeight", "netto");
        $rate=$this->getRateFromCache($ShippingInfo,$pak_weight,$method_id);
        if($rate!=false)
            return $rate;
        else
            return PRICE_N_A;
    }


    var $bActive;

    var $is_debug;
    var $_debug_info;

    /**#@-*/
}
?>